@@ -170,15 +170,18 @@
       me.email_submit = $('#civicrm_email_submit');
       me.permissions_submit = $('#civicrm_permissions_submit');
       me.cache_submit = $('#civicrm_cache_submit');
+      me.auto_sign_in_user_submit = $('#civicrm_auto_sign_in_user_submit');
       me.basepage_select = $('#page_id');
       me.shortcode_select = $('#shortcode_mode');
       me.theme_select = $('#theme_compatibility_mode');
       me.email_select = $('#sync_email');
       me.permissions_select = $('#permissions_role');
+      me.auto_sign_in_user_select = $('#auto_sign_in_user');
       me.basepage_selected = me.basepage_select.val();
       me.shortcode_selected = me.shortcode_select.val();
       me.theme_selected = me.theme_select.val();
       me.email_selected = me.email_select.val();
+      me.auto_sign_in_user_selected = me.auto_sign_in_user_select.val();
 
       // Set status of Base Page submit button.
       me.basepage_submit.prop('disabled', true);
@@ -198,6 +201,10 @@
       me.email_submit.val(text);
       me.email_submit.prop('disabled', true);
 
+      // Set status of the Automatically Sign In User submit button.
+      me.auto_sign_in_user_submit.val(text);
+      me.auto_sign_in_user_submit.prop('disabled', true);
+
     };
 
     /**
@@ -305,6 +312,27 @@
 
       });
 
+      /**
+       * Add an onchange event listener to the "Automatically Sign In User" section select.
+       *
+       * @param {Object} event The event object.
+       */
+      me.auto_sign_in_user_select.on('change', function(event) {
+
+        var text;
+
+        // Enable/disable submit button.
+        if (me.auto_sign_in_user_select.val() == me.auto_sign_in_user_selected) {
+          text = CiviCRM_Options_Settings.get_localisation('saved');
+          me.auto_sign_in_user_submit.val(text);
+          me.auto_sign_in_user_submit.prop('disabled', true);
+        } else {
+          text = CiviCRM_Options_Settings.get_localisation('update');
+          me.auto_sign_in_user_submit.val(text);
+          me.auto_sign_in_user_submit.prop('disabled', false);
+        }
+      });
+
       /**
        * Add a click event listener to the "Basepage" section submit button.
        *
@@ -314,8 +342,8 @@
 
         // Define vars.
         var value = me.basepage_select.val(),
-            ajax_nonce = me.basepage_submit.data('security'),
-            saving = CiviCRM_Options_Settings.get_localisation('saving');
+          ajax_nonce = me.basepage_submit.data('security'),
+          saving = CiviCRM_Options_Settings.get_localisation('saving');
 
         // Prevent form submission.
         if (event.preventDefault) {
@@ -342,8 +370,8 @@
 
         // Define vars.
         var value = me.shortcode_select.val(),
-            ajax_nonce = me.shortcode_submit.data('security'),
-            saving = CiviCRM_Options_Settings.get_localisation('saving');
+          ajax_nonce = me.shortcode_submit.data('security'),
+          saving = CiviCRM_Options_Settings.get_localisation('saving');
 
         // Prevent form submission.
         if (event.preventDefault) {
@@ -370,8 +398,8 @@
 
         // Define vars.
         var value = me.theme_select.val(),
-            ajax_nonce = me.theme_submit.data('security'),
-            saving = CiviCRM_Options_Settings.get_localisation('saving');
+          ajax_nonce = me.theme_submit.data('security'),
+          saving = CiviCRM_Options_Settings.get_localisation('saving');
 
         // Prevent form submission.
         if (event.preventDefault) {
@@ -398,8 +426,8 @@
 
         // Define vars.
         var value = me.email_select.val(),
-            ajax_nonce = me.email_submit.data('security'),
-            saving = CiviCRM_Options_Settings.get_localisation('saving');
+          ajax_nonce = me.email_submit.data('security'),
+          saving = CiviCRM_Options_Settings.get_localisation('saving');
 
         // Prevent form submission.
         if (event.preventDefault) {
@@ -426,8 +454,8 @@
 
         // Define vars.
         var value = me.permissions_select.val(),
-            ajax_nonce = me.permissions_submit.data('security'),
-            refreshing = CiviCRM_Options_Settings.get_localisation('refreshing');
+          ajax_nonce = me.permissions_submit.data('security'),
+          refreshing = CiviCRM_Options_Settings.get_localisation('refreshing');
 
         // Prevent form submission.
         if (event.preventDefault) {
@@ -454,7 +482,7 @@
 
         // Define vars.
         var ajax_nonce = me.cache_submit.data('security'),
-            clearing = CiviCRM_Options_Settings.get_localisation('clearing');
+          clearing = CiviCRM_Options_Settings.get_localisation('clearing');
 
         // Prevent form submission.
         if (event.preventDefault) {
@@ -471,6 +499,34 @@
 
       });
 
+      /**
+       * Add a click event listener to the "Automatically Sign In User" section submit button.
+       *
+       * @param {Object} event The event object.
+       */
+      me.auto_sign_in_user_submit.on('click', function(event) {
+
+        // Define vars.
+        var value = me.auto_sign_in_user_select.val(),
+          ajax_nonce = me.auto_sign_in_user_submit.data('security'),
+          saving = CiviCRM_Options_Settings.get_localisation('saving');
+
+        // Prevent form submission.
+        if (event.preventDefault) {
+          event.preventDefault();
+        }
+
+        // Modify button and select, then show spinner.
+        me.auto_sign_in_user_submit.val(saving);
+        me.auto_sign_in_user_submit.prop('disabled', true);
+        me.auto_sign_in_user_select.prop('disabled', true);
+        $(this).next('.spinner').css('visibility', 'visible');
+
+        // Submit request to server.
+        me.send('civicrm_auto_sign_in_user', value, ajax_nonce);
+
+      });
+
     };
 
     /**
@@ -538,14 +594,14 @@
 
       // Declare vars.
       var saved = CiviCRM_Options_Settings.get_localisation('saved'),
-          update = CiviCRM_Options_Settings.get_localisation('update'),
-          clearing = CiviCRM_Options_Settings.get_localisation('clearing'),
-          refresh = CiviCRM_Options_Settings.get_localisation('refresh'),
-          cache = CiviCRM_Options_Settings.get_localisation('cache');
+        update = CiviCRM_Options_Settings.get_localisation('update'),
+        clearing = CiviCRM_Options_Settings.get_localisation('clearing'),
+        refresh = CiviCRM_Options_Settings.get_localisation('refresh'),
+        cache = CiviCRM_Options_Settings.get_localisation('cache');
 
       if (data.saved) {
 
-		// Success!
+        // Success!
         if (data.section == 'basepage') {
 
           // Base Page section.
@@ -570,7 +626,7 @@
 
         } else if (data.section == 'theme') {
 
-          // Shortcode Mode section.
+          // Shortcode Theme Compatibility section.
           me.theme_submit.val(saved);
           me.theme_submit.next('.spinner').css('visibility', 'hidden');
           me.theme_select.val(data.result);
@@ -607,11 +663,20 @@
           me.cache_submit.prop('disabled', false);
           me.cache_submit.next('.spinner').css('visibility', 'hidden');
 
+        } else if (data.section == 'auto_sign_in_user') {
+
+          // Automatically Sign In User section.
+          me.auto_sign_in_user_submit.val(saved);
+          me.auto_sign_in_user_submit.next('.spinner').css('visibility', 'hidden');
+          me.auto_sign_in_user_select.val(data.result);
+          me.auto_sign_in_user_selected = data.result;
+          me.auto_sign_in_user_select.prop('disabled', false);
+
         }
 
       } else {
 
-		// Failure.
+        // Failure.
         if (data.section == 'basepage') {
 
           // Base Page section.
@@ -637,7 +702,7 @@
 
         } else if (data.section == 'theme') {
 
-          // Shortcode Mode section.
+          // Shortcode Theme Compatibility section.
           me.theme_submit.val(update);
           me.theme_submit.next('.spinner').css('visibility', 'hidden');
           me.theme_select.val(data.result);
@@ -678,6 +743,17 @@
           $('.caches_error').show();
           $('.caches_error p').html(data.notice);
 
+        } else if (data.section == 'auto_sign_in_user') {
+
+          // Automatically Sign In User section.
+          me.auto_sign_in_user_submit.val(update);
+          me.auto_sign_in_user_submit.next('.spinner').css('visibility', 'hidden');
+          me.auto_sign_in_user_select.val(data.result);
+          me.auto_sign_in_user_select.prop('disabled', false);
+          $('.auto_sign_in_user_notice').show();
+          $('.auto_sign_in_user_notice p').html(data.notice);
+          me.auto_sign_in_user_selected = data.result;
+
         }
 
       }
