@@ -165,7 +165,7 @@ WHERE  id IN ( $idString )
       // to avoid breaking things elsewhere
       // See lab issue #5541
       do {
-        $hash = md5(uniqid(rand(), TRUE));
+        $hash = bin2hex(random_bytes(16));
         if ($hashSize) {
           $hash = substr($hash, 0, $hashSize);
         }
@@ -461,83 +461,6 @@ WHERE id={$contactId}; ";
     }
   }
 
-  /**
-   * Build form for related contacts / on behalf of organization.
-   *
-   * @param CRM_Core_Form $form
-   * @param string $contactType
-   *   contact type.
-   * @param int $countryID
-   * @param int $stateID
-   * @param string $title
-   *   fieldset title.
-   *
-   * @deprecated since 5.74 will be removed around 5.80
-   *
-   * @throws \CRM_Core_Exception
-   */
-  public static function buildOnBehalfForm(&$form, $contactType, $countryID, $stateID, $title) {
-    CRM_Core_Error::deprecatedFunctionWarning('no alternative');
-    $form->assign('contact_type', $contactType);
-    $form->assign('fieldSetTitle', $title);
-    $form->assign('contactEditMode', TRUE);
-
-    $attributes = CRM_Core_DAO::getAttribute('CRM_Contact_DAO_Contact');
-    if ($form->_contactId) {
-      $form->assign('orgId', $form->_contactId);
-    }
-
-    switch ($contactType) {
-      case 'Organization':
-        $form->add('text', 'organization_name', ts('Organization Name'), $attributes['organization_name'], TRUE);
-        break;
-
-      case 'Household':
-        $form->add('text', 'household_name', ts('Household Name'), $attributes['household_name']);
-        break;
-
-      default:
-        // individual
-        $form->addElement('select', 'prefix_id', ts('Prefix'),
-          ['' => ts('- prefix -')] + CRM_Contact_DAO_Contact::buildOptions('prefix_id')
-        );
-        $form->addElement('text', 'first_name', ts('First Name'),
-          $attributes['first_name']
-        );
-        $form->addElement('text', 'middle_name', ts('Middle Name'),
-          $attributes['middle_name']
-        );
-        $form->addElement('text', 'last_name', ts('Last Name'),
-          $attributes['last_name']
-        );
-        $form->addElement('select', 'suffix_id', ts('Suffix'),
-          ['' => ts('- suffix -')] + CRM_Contact_DAO_Contact::buildOptions('suffix_id')
-        );
-    }
-
-    $addressSequence = CRM_Utils_Address::sequence(\Civi::settings()->get('address_format'));
-    $form->assign('addressSequence', array_fill_keys($addressSequence, 1));
-
-    //Primary Phone
-    $form->addElement('text',
-      'phone[1][phone]',
-      ts('Primary Phone'),
-      CRM_Core_DAO::getAttribute('CRM_Core_DAO_Phone',
-        'phone'
-      )
-    );
-    //Primary Email
-    $form->addElement('text',
-      'email[1][email]',
-      ts('Primary Email'),
-      CRM_Core_DAO::getAttribute('CRM_Core_DAO_Email',
-        'email'
-      )
-    );
-    //build the address block
-    CRM_Contact_Form_Edit_Address::buildQuickForm($form);
-  }
-
   /**
    * Clear cache employer name and employer id
    * of all employee when employer get deleted.
