@@ -9,14 +9,11 @@
  +--------------------------------------------------------------------+
  */
 
-use Civi\Api4\Contact;
 use Civi\Api4\County;
 use Civi\Api4\RelationshipType;
 use Civi\Api4\StateProvince;
 use Civi\Api4\DedupeRuleGroup;
 
-require_once 'api/v3/utils.php';
-
 /**
  *
  * @package CRM
@@ -30,23 +27,7 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
 
   private $externalIdentifiers = [];
 
-  /**
-   * Array of successfully imported contact id's
-   *
-   * @var array
-   */
-  protected $_newContacts = [];
-
-  protected $_tableName;
-
-  /**
-   * Total number of lines in file
-   *
-   * @var int
-   */
-  protected $_rowCount;
-
-  protected $fieldMetadata = [];
+  protected $baseEntity = 'Contact';
 
   /**
    * Relationship labels.
@@ -131,6 +112,7 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
    *   The array of values belonging to this line.
    */
   public function import(array $values): void {
+    $values = array_values($values);
     $rowNumber = (int) $values[array_key_last($values)];
 
     // Put this here for now since we're gettting run by a job and need to
@@ -146,6 +128,7 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
 
     try {
       $params = $this->getMappedRow($values);
+      $params['id'] = $this->processContact($params, TRUE);
       $formatted = [];
       foreach ($params as $key => $value) {
         if ($value !== '') {
@@ -153,24 +136,12 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
         }
       }
 
-      $formatted['id'] = $params['id'] = $this->processContact($params, TRUE);
-
       //format common data, CRM-4062
-      $this->formatCommonData($params, $formatted);
+      $this->augmentAddressData($formatted);
 
       $newContact = $this->createContact($formatted, $params['id'] ?? NULL);
       $contactID = $newContact->id;
 
-      if ($contactID) {
-        // call import hook
-        $currentImportID = end($values);
-        $hookParams = [
-          'contactID' => $contactID,
-          'importID' => $currentImportID,
-        ];
-        CRM_Utils_Hook::import('Contact', 'process', $this, $hookParams);
-      }
-
       $primaryContactId = $newContact->id;
 
       //relationship contact insert
@@ -178,7 +149,7 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
         $field['id'] = $this->processContact($field, FALSE);
         $formatting = $field;
         //format common data, CRM-4062
-        $this->formatCommonData($field, $formatting);
+        $this->augmentAddressData($formatting);
         $isUpdate = empty($formatting['id']) ? 'new' : 'updated';
         if (empty($formatting['id']) || $this->isUpdateExisting()) {
           $relatedNewContact = $this->createContact($formatting, $formatting['id']);
@@ -204,75 +175,10 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
         $extraFields['related_contact_matched']++;
       }
     }
+    $this->callLegacyHook($contactID ?? NULL, $values);
     $this->setImportStatus($rowNumber, $this->getStatus(CRM_Import_Parser::VALID), $this->getSuccessMessage(), $contactID, $extraFields, array_merge(array_keys($relatedContacts), [$contactID]));
   }
 
-  /**
-   * Only called from import now... plus one place outside of core & tests.
-   *
-   * @todo - deprecate more aggressively - will involve copying to the import
-   * class, adding a deprecation notice here & removing from tests.
-   *
-   * Takes an associative array and creates a relationship object.
-   *
-   * @deprecated For single creates use the api instead (it's tested).
-   * For multiple a new variant of this function needs to be written and migrated to as this is a bit
-   * nasty
-   *
-   * @param array $params
-   *   (reference ) an assoc array of name/value pairs.
-   * @param array $ids
-   *   The array that holds all the db ids.
-   *   per http://wiki.civicrm.org/confluence/display/CRM/Database+layer
-   *  "we are moving away from the $ids param "
-   *
-   * @return array
-   * @throws \CRM_Core_Exception
-   */
-  private static function legacyCreateMultiple($params, $ids = []) {
-    // clarify that the only key ever pass in the ids array is 'contact'
-    // There is legacy handling for other keys but a universe search on
-    // calls to this function (not supported to be called from outside core)
-    // only returns 2 calls - one in CRM_Contact_Import_Parser_Contact
-    // and the other in jma grant applications (CRM_Grant_Form_Grant_Confirm)
-    // both only pass in contact as a key here.
-    $contactID = $ids['contact'];
-    unset($ids);
-    // There is only ever one value passed in from the 2 places above that call
-    // this - by clarifying here like this we can cleanup within this
-    // function without having to do more universe searches.
-    $relatedContactID = key($params['contact_check']);
-
-    // check if the relationship is valid between contacts.
-    // step 1: check if the relationship is valid if not valid skip and keep the count
-    // step 2: check the if two contacts already have a relationship if yes skip and keep the count
-    // step 3: if valid relationship then add the relation and keep the count
-
-    // step 1
-    [$contactFields['relationship_type_id'], $firstLetter, $secondLetter] = explode('_', $params['relationship_type_id']);
-    $contactFields['contact_id_' . $firstLetter] = $contactID;
-    $contactFields['contact_id_' . $secondLetter] = $relatedContactID;
-    if (!CRM_Contact_BAO_Relationship::checkRelationshipType($contactFields['contact_id_a'], $contactFields['contact_id_b'],
-      $contactFields['relationship_type_id'])) {
-      return [0, 0];
-    }
-
-    if (
-      CRM_Contact_BAO_Relationship::checkDuplicateRelationship(
-        $contactFields,
-        (int) $contactID,
-        // step 2
-        (int) $relatedContactID
-      )
-    ) {
-      return [0, 1];
-    }
-
-    $singleInstanceParams = array_merge($params, $contactFields);
-    CRM_Contact_BAO_Relationship::add($singleInstanceParams);
-    return [1, 0];
-  }
-
   /**
    * Format common params data to the format that was required a very long time ago.
    *
@@ -280,41 +186,15 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
    *  1) calls fillPrimary
    *  2) possibly the street address parsing.
    *
-   * The other hundred lines do stuff that is done elsewhere. Custom fields
-   * should already be formatted by getTransformedValue and we don't need to
-   * re-rewrite them to a BAO style array since we call the api which does that.
+   * The other hundred lines do stuff that is done elsewhere.
    *
    * The call to formatLocationBlock just does the address custom fields which,
    * are already formatted by this point.
    *
-   * @deprecated
-   *
-   * @param array $params
-   *   Contain record values.
    * @param array $formatted
    *   Array of formatted data.
    */
-  private function formatCommonData($params, &$formatted) {
-    // @todo - remove just about everything in this function. See docblock.
-    $customFields = CRM_Core_BAO_CustomField::getFields($formatted['contact_type'], FALSE, FALSE, $formatted['contact_sub_type'] ?? NULL);
-
-    $addressCustomFields = CRM_Core_BAO_CustomField::getFields('Address');
-    $customFields = $customFields + $addressCustomFields;
-
-    foreach ($params as $key => $val) {
-      $customFieldID = CRM_Core_BAO_CustomField::getKeyID($key);
-      if ($customFieldID &&
-        !array_key_exists($customFieldID, $addressCustomFields)
-      ) {
-        // @todo - this can probably go....
-        if ($customFields[$customFieldID]['data_type'] === 'Boolean') {
-          if (empty($val) && !is_numeric($val) && $this->isFillDuplicates()) {
-            //retain earlier value when Import mode is `Fill`
-            unset($params[$key]);
-          }
-        }
-      }
-    }
+  private function augmentAddressData(array &$formatted): void {
     $metadataBlocks = ['phone', 'im', 'openid', 'email', 'address'];
     foreach ($metadataBlocks as $block) {
       foreach ($formatted[$block] ?? [] as $blockKey => $blockValues) {
@@ -323,101 +203,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
         }
       }
     }
-    //now format custom data.
-    foreach ($params as $key => $field) {
-      if (in_array($key, $metadataBlocks, TRUE)) {
-        // This location block is already fully handled at this point.
-        continue;
-      }
-      if (is_array($field)) {
-        $isAddressCustomField = FALSE;
-
-        foreach ($field as $value) {
-          $break = FALSE;
-          if (is_array($value)) {
-            foreach ($value as $name => $testForEmpty) {
-              if ($addressCustomFieldID = CRM_Core_BAO_CustomField::getKeyID($name)) {
-                $isAddressCustomField = TRUE;
-                break;
-              }
-
-              if (($testForEmpty === '' || $testForEmpty == NULL)) {
-                $break = TRUE;
-                break;
-              }
-            }
-          }
-          else {
-            $break = TRUE;
-          }
-
-          if (!$break) {
-            if (!empty($value['location_type_id'])) {
-              $this->formatLocationBlock($value, $formatted);
-            }
-          }
-        }
-        if (!$isAddressCustomField) {
-          continue;
-        }
-      }
-
-      $formatValues = [
-        $key => $field,
-      ];
-
-      if ($key == 'id' && isset($field)) {
-        $formatted[$key] = $field;
-      }
-      $this->formatContactParameters($formatValues, $formatted);
-
-      //Handling Custom Data
-      // note: Address custom fields will be handled separately inside formatContactParameters
-      if (($customFieldID = CRM_Core_BAO_CustomField::getKeyID($key)) &&
-        array_key_exists($customFieldID, $customFields) &&
-        !array_key_exists($customFieldID, $addressCustomFields)
-      ) {
-
-        $extends = $customFields[$customFieldID]['extends'] ?? NULL;
-        $htmlType = $customFields[$customFieldID]['html_type'] ?? NULL;
-        $dataType = $customFields[$customFieldID]['data_type'] ?? NULL;
-        $serialized = CRM_Core_BAO_CustomField::isSerialized($customFields[$customFieldID]);
-
-        if (!$serialized && in_array($htmlType, ['Select', 'Radio', 'Autocomplete-Select']) && in_array($dataType, ['String', 'Int'])) {
-          $customOption = CRM_Core_BAO_CustomOption::getCustomOption($customFieldID, TRUE);
-          foreach ($customOption as $customValue) {
-            $val = $customValue['value'] ?? NULL;
-            $label = strtolower($customValue['label'] ?? '');
-            $value = strtolower(trim($formatted[$key] ?? ''));
-            if (($value == $label) || ($value == strtolower($val))) {
-              $params[$key] = $formatted[$key] = $val;
-            }
-          }
-        }
-        elseif ($serialized && !empty($formatted[$key]) && !empty($params[$key])) {
-          $mulValues = explode(',', $formatted[$key]);
-          $customOption = CRM_Core_BAO_CustomOption::getCustomOption($customFieldID, TRUE);
-          $formatted[$key] = [];
-          $params[$key] = [];
-          foreach ($mulValues as $v1) {
-            foreach ($customOption as $v2) {
-              if ((strtolower($v2['label']) == strtolower(trim($v1))) ||
-                (strtolower($v2['value']) == strtolower(trim($v1)))
-              ) {
-                $params[$key][] = $formatted[$key][] = $v2['value'];
-              }
-            }
-          }
-        }
-      }
-    }
-
-    if (!empty($key) && ($customFieldID = CRM_Core_BAO_CustomField::getKeyID($key)) && array_key_exists($customFieldID, $customFields) &&
-      !array_key_exists($customFieldID, $addressCustomFields)
-    ) {
-      // @todo calling api functions directly is not supported
-      _civicrm_api3_custom_format_params($params, $formatted, $extends);
-    }
 
     // parse street address, CRM-5450
     if ($this->isParseStreetAddress()) {
@@ -443,28 +228,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
     }
   }
 
-  /**
-   * Build error-message containing error-fields
-   *
-   * Once upon a time there was a dev who hadn't heard of implode. That dev wrote this function.
-   *
-   * @todo just say no!
-   *
-   * @param string $errorName
-   *   A string containing error-field name.
-   * @param string $errorMessage
-   *   A string containing all the error-fields, where the new errorName is concatenated.
-   *
-   */
-  public static function addToErrorMsg($errorName, &$errorMessage) {
-    if ($errorMessage) {
-      $errorMessage .= "; $errorName";
-    }
-    else {
-      $errorMessage = $errorName;
-    }
-  }
-
   /**
    * Get sorted available relationships.
    *
@@ -485,6 +248,24 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
     return $relations;
   }
 
+  /**
+   * Call legacy, strongly discouraged, hook.
+   *
+   * @param int|string|null $contactID
+   * @param array $values
+   */
+  private function callLegacyHook(int|string|null $contactID, array $values): void {
+    if ($contactID) {
+      // call import hook
+      $currentImportID = end($values);
+      $hookParams = [
+        'contactID' => $contactID,
+        'importID' => $currentImportID,
+      ];
+      CRM_Utils_Hook::import('Contact', 'process', $this, $hookParams);
+    }
+  }
+
   /**
    * @param array $params
    *
@@ -527,64 +308,48 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
   }
 
   /**
-   * @param $key
-   * @param $relContactId
-   * @param $primaryContactId
+   * @param string $key
+   * @param int $relatedContactID
+   * @param int $primaryContactId
    *
    * @throws \CRM_Core_Exception
    */
-  protected function createRelationship($key, $relContactId, $primaryContactId): void {
-    //if more than one duplicate contact
-    //found, create relationship with first contact
-    // now create the relationship record
-    $relationParams = [
-      'relationship_type_id' => $key,
-      'contact_check' => [
-        $relContactId => 1,
-      ],
+  protected function createRelationship(string $key, int $relatedContactID, int $primaryContactId): void {
+    // check if the relationship is valid between contacts.
+    // step 1: check if the relationship is valid if not valid skip.
+    // step 2: check the if two contacts already have a relationship if yes skip.
+    // step 3: if valid relationship then add the relation.
+
+    [$contactFields['relationship_type_id'], $firstLetter, $secondLetter] = explode('_', $key);
+    $contactFields['contact_id_' . $firstLetter] = $primaryContactId;
+    $contactFields['contact_id_' . $secondLetter] = $relatedContactID;
+    if (!CRM_Contact_BAO_Relationship::checkRelationshipType($contactFields['contact_id_a'], $contactFields['contact_id_b'],
+      $contactFields['relationship_type_id'])) {
+      return;
+    }
+
+    if (
+      CRM_Contact_BAO_Relationship::checkDuplicateRelationship(
+        $contactFields,
+        (int) $primaryContactId,
+        // step 2
+        (int) $relatedContactID
+      )
+    ) {
+      return;
+    }
+
+    $singleInstanceParams = array_merge([
       'is_active' => 1,
       'skipRecentView' => TRUE,
-    ];
-
-    // we only handle related contact success, we ignore failures for now
-    // at some point wold be nice to have related counts as separate
-    $relationIds = [
-      'contact' => $primaryContactId,
-    ];
-
-    [$valid, $duplicate] = self::legacyCreateMultiple($relationParams, $relationIds);
-
-    if ($valid || $duplicate) {
-      $relationIds['contactTarget'] = $relContactId;
-      $action = ($duplicate) ? CRM_Core_Action::UPDATE : CRM_Core_Action::ADD;
-      CRM_Contact_BAO_Relationship::relatedMemberships($primaryContactId, $relationParams, $relationIds, $action);
-    }
-
-    //handle current employer, CRM-3532
-    if ($valid) {
-      $allRelationships = CRM_Core_PseudoConstant::relationshipType('name');
-      $relationshipTypeId = str_replace([
-        '_a_b',
-        '_b_a',
-      ], [
-        '',
-        '',
-      ], $key);
-      $relationshipType = str_replace($relationshipTypeId . '_', '', $key);
-      $orgId = $individualId = NULL;
-      if ($allRelationships[$relationshipTypeId]["name_{$relationshipType}"] == 'Employee of') {
-        $orgId = $relContactId;
-        $individualId = $primaryContactId;
-      }
-      elseif ($allRelationships[$relationshipTypeId]["name_{$relationshipType}"] == 'Employer of') {
-        $orgId = $primaryContactId;
-        $individualId = $relContactId;
-      }
-      if ($orgId && $individualId) {
-        $currentEmpParams[$individualId] = $orgId;
-        CRM_Contact_BAO_Contact_Utils::setCurrentEmployer($currentEmpParams);
-      }
-    }
+    ], $contactFields);
+    // Setting is_current_employer means that IF the relationship is an employment one
+    // employee_id & organization_name will be updated on the individual.
+    // This happens in the BAO_Relationship::add function.
+    // If it were not for needing to pass this parameter we could use
+    // apiv4 here instead.
+    $singleInstanceParams['is_current_employer'] = TRUE;
+    CRM_Contact_BAO_Relationship::create($singleInstanceParams);
   }
 
   /**
@@ -595,47 +360,29 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
    *
    * @return \CRM_Contact_BAO_Contact
    *   If a duplicate is found an array is returned, otherwise CRM_Contact_BAO_Contact
+   * @throws \CRM_Core_Exception
    */
   public function createContact(&$formatted, $contactId = NULL) {
 
     if ($contactId) {
+      $formatted['id'] = $contactId;
       $this->formatParams($formatted, (int) $contactId);
+      // manage is_opt_out
+      $existingOptOut = $this->getExistingContactValue($contactId, 'is_opt_out');
+      if (array_key_exists('is_opt_out', $formatted) && $existingOptOut !== (bool) $formatted['is_opt_out']
+      ) {
+        // on change, create new civicrm_subscription_history entry
+        CRM_Contact_BAO_SubscriptionHistory::writeRecord([
+          'contact_id' => $contactId,
+          'status' => $formatted['is_opt_out'] ? 'Removed' : 'Added',
+          'method' => 'Import',
+        ]);
+      }
     }
 
     // Resetting and rebuilding cache could be expensive.
     CRM_Core_Config::setPermitCacheFlushMode(FALSE);
-
-    // If a user has logged in, or accessed via a checksum
-    // Then deliberately 'blanking' a value in the profile should remove it from their record
-    // @todo this should either be TRUE or FALSE in the context of import - once
-    // we figure out which we can remove all the rest.
-    // Also note the meaning of this parameter is less than it used to
-    // be following block cleanup.
-    $formatted['updateBlankLocInfo'] = TRUE;
-    if ((CRM_Core_Session::singleton()->get('authSrc') & (CRM_Core_Permission::AUTH_SRC_CHECKSUM + CRM_Core_Permission::AUTH_SRC_LOGIN)) == 0) {
-      $formatted['updateBlankLocInfo'] = FALSE;
-    }
-
-    $contactFields = CRM_Contact_DAO_Contact::import();
-    [$data, $contactDetails] = $this->formatProfileContactParams($formatted, $contactFields, $contactId, $formatted['contact_type']);
-
-    // manage is_opt_out
-    if (array_key_exists('is_opt_out', $contactFields) && array_key_exists('is_opt_out', $formatted)) {
-      $wasOptOut = $contactDetails['is_opt_out'] ?? FALSE;
-      $isOptOut = $formatted['is_opt_out'];
-      $data['is_opt_out'] = $isOptOut;
-      // on change, create new civicrm_subscription_history entry
-      if (($wasOptOut != $isOptOut) && !empty($contactDetails['contact_id'])) {
-        $shParams = [
-          'contact_id' => $contactDetails['contact_id'],
-          'status' => $isOptOut ? 'Removed' : 'Added',
-          'method' => 'Web',
-        ];
-        CRM_Contact_BAO_SubscriptionHistory::create($shParams);
-      }
-    }
-
-    $contact = civicrm_api3('Contact', 'create', $data);
+    $contact = civicrm_api3('Contact', 'create', $formatted);
     $cid = $contact['id'];
 
     CRM_Core_Config::setPermitCacheFlushMode(TRUE);
@@ -661,218 +408,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
     return $newContact;
   }
 
-  /**
-   * Legacy format profile contact parameters.
-   *
-   * This is a formerly shared function - most of the stuff in it probably does
-   * nothing but copied here to star unravelling that...
-   *
-   * @param array $params
-   * @param array $fields
-   * @param int|null $contactID
-   * @param string|null $ctype
-   *
-   * @return array
-   */
-  private function formatProfileContactParams(
-    &$params,
-    $fields,
-    $contactID = NULL,
-    $ctype = NULL
-  ) {
-
-    $data = $contactDetails = [];
-
-    // get the contact details (hier)
-    if ($contactID) {
-      $details = CRM_Contact_BAO_Contact::getHierContactDetails($contactID, $fields);
-
-      $contactDetails = $details[$contactID];
-      $data['contact_type'] = $contactDetails['contact_type'] ?? NULL;
-      $data['contact_sub_type'] = $contactDetails['contact_sub_type'] ?? NULL;
-    }
-    else {
-      //we should get contact type only if contact
-      if ($ctype) {
-        $data['contact_type'] = $ctype;
-      }
-      else {
-        $data['contact_type'] = 'Individual';
-      }
-    }
-
-    //fix contact sub type CRM-5125
-    if (array_key_exists('contact_sub_type', $params) &&
-      !empty($params['contact_sub_type'])
-    ) {
-      $data['contact_sub_type'] = CRM_Utils_Array::implodePadded($params['contact_sub_type']);
-    }
-    elseif (array_key_exists('contact_sub_type_hidden', $params) &&
-      !empty($params['contact_sub_type_hidden'])
-    ) {
-      // if profile was used, and had any subtype, we obtain it from there
-      //CRM-13596 - add to existing contact types, rather than overwriting
-      if (empty($data['contact_sub_type'])) {
-        // If we don't have a contact ID the $data['contact_sub_type'] will not be defined...
-        $data['contact_sub_type'] = CRM_Utils_Array::implodePadded($params['contact_sub_type_hidden']);
-      }
-      else {
-        $data_contact_sub_type_arr = CRM_Utils_Array::explodePadded($data['contact_sub_type']);
-        if (!in_array($params['contact_sub_type_hidden'], $data_contact_sub_type_arr)) {
-          //CRM-20517 - make sure contact_sub_type gets the correct delimiters
-          $data['contact_sub_type'] = trim($data['contact_sub_type'], CRM_Core_DAO::VALUE_SEPARATOR);
-          $data['contact_sub_type'] = CRM_Core_DAO::VALUE_SEPARATOR . $data['contact_sub_type'] . CRM_Utils_Array::implodePadded($params['contact_sub_type_hidden']);
-        }
-      }
-    }
-
-    if ($ctype == 'Organization') {
-      $data['organization_name'] = $contactDetails['organization_name'] ?? NULL;
-    }
-    elseif ($ctype == 'Household') {
-      $data['household_name'] = $contactDetails['household_name'] ?? NULL;
-    }
-
-    $locationType = [];
-    $count = 1;
-
-    if ($contactID) {
-      //add contact id
-      $data['contact_id'] = $contactID;
-      $primaryLocationType = CRM_Contact_BAO_Contact::getPrimaryLocationType($contactID);
-    }
-    else {
-      $defaultLocation = CRM_Core_BAO_LocationType::getDefault();
-      $defaultLocationId = $defaultLocation->id;
-    }
-
-    $billingLocationTypeId = CRM_Core_BAO_LocationType::getBilling();
-
-    $multiplFields = ['url'];
-
-    $session = CRM_Core_Session::singleton();
-    foreach ($params as $key => $value) {
-      [$fieldName, $locTypeId, $typeId] = CRM_Utils_System::explode('-', $key, 3);
-
-      if ($locTypeId == 'Primary') {
-        if ($contactID) {
-          $locTypeId = CRM_Contact_BAO_Contact::getPrimaryLocationType($contactID, FALSE, 'address');
-          $primaryLocationType = $locTypeId;
-        }
-        else {
-          $locTypeId = $defaultLocationId;
-        }
-      }
-
-      if (is_numeric($locTypeId) &&
-        !in_array($fieldName, $multiplFields) &&
-        substr($fieldName, 0, 7) !== 'custom_'
-      ) {
-        $index = $locTypeId;
-
-        if (is_numeric($typeId)) {
-          $index .= '-' . $typeId;
-        }
-        if (!in_array($index, $locationType)) {
-          $locationType[$count] = $index;
-          $count++;
-        }
-
-        $loc = CRM_Utils_Array::key($index, $locationType);
-
-        $blockName = strtolower($this->getFieldEntity($fieldName));
-
-        $data[$blockName][$loc]['location_type_id'] = $locTypeId;
-
-        //set is_billing true, for location type "Billing"
-        if ($locTypeId == $billingLocationTypeId) {
-          $data[$blockName][$loc]['is_billing'] = 1;
-        }
-
-        if ($contactID) {
-          //get the primary location type
-          if ($locTypeId == $primaryLocationType) {
-            $data[$blockName][$loc]['is_primary'] = 1;
-          }
-        }
-        elseif ($locTypeId == $defaultLocationId) {
-          $data[$blockName][$loc]['is_primary'] = 1;
-        }
-
-        if ($fieldName === 'state_province') {
-          // CRM-3393
-          if (is_numeric($value) && ((int ) $value) >= 1000) {
-            $data['address'][$loc]['state_province_id'] = $value;
-          }
-          elseif (empty($value)) {
-            $data['address'][$loc]['state_province_id'] = '';
-          }
-          else {
-            $data['address'][$loc]['state_province'] = $value;
-          }
-        }
-        elseif ($fieldName === 'country_id') {
-          $data['address'][$loc]['country_id'] = $value;
-        }
-        elseif ($fieldName === 'county') {
-          $data['address'][$loc]['county_id'] = $value;
-        }
-        elseif ($fieldName == 'address_name') {
-          $data['address'][$loc]['name'] = $value;
-        }
-        elseif (substr($fieldName, 0, 14) === 'address_custom') {
-          $data['address'][$loc][substr($fieldName, 8)] = $value;
-        }
-        else {
-          $data[$blockName][$loc][$fieldName] = $value;
-        }
-      }
-
-      if ($key === 'location') {
-        foreach ($value as $locationTypeId => $field) {
-          foreach ($field as $block => $val) {
-            if ($block === 'address' && array_key_exists('address_name', $val)) {
-              $value[$locationTypeId][$block]['name'] = $value[$locationTypeId][$block]['address_name'];
-            }
-          }
-        }
-      }
-      // Why only these fields...?
-      if ($value === '' && in_array($key, ['nick_name', 'job_title', 'middle_name', 'birth_date', 'gender_id', 'current_employer', 'prefix_id', 'suffix_id'], TRUE)
-        ) {
-        // CRM-10128: if $value is blank, do not fill $data with empty value
-        continue;
-      }
-      else {
-        $data[$key] = $value;
-      }
-
-    }
-
-    if (!isset($data['contact_type'])) {
-      $data['contact_type'] = 'Individual';
-    }
-
-    //set the values for checkboxes (do_not_email, do_not_mail, do_not_trade, do_not_phone)
-    $privacy = CRM_Core_SelectValues::privacy();
-    foreach ($privacy as $key => $value) {
-      if (array_key_exists($key, $fields)) {
-        // do not reset values for existing contacts, if fields are added to a profile
-        if (array_key_exists($key, $params)) {
-          $data[$key] = $params[$key];
-          if (empty($params[$key])) {
-            $data[$key] = 0;
-          }
-        }
-        elseif (!$contactID) {
-          $data[$key] = 0;
-        }
-      }
-    }
-
-    return [$data, $contactDetails];
-  }
-
   /**
    * Format params for update and fill mode.
    *
@@ -887,67 +422,53 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
       return;
     }
 
-    $contactParams = [
-      'contact_id' => $cid,
-      // core#4269 - Don't check relationships for values.
-      'noRelationships' => TRUE,
+    $contactObj = new CRM_Contact_DAO_Contact();
+    $contactObj->id = $cid;
+    $contactObj->find(TRUE);
+    $blocks = [
+      'email' => CRM_Core_BAO_Email::getValues(['contact_id' => $cid]),
+      'phone' => CRM_Core_BAO_Phone::getValues(['contact_id' => $cid]),
+      'address' => CRM_Core_BAO_Address::getValues(['contact_id' => $cid]),
     ];
-
-    $defaults = [];
-    $contactObj = CRM_Contact_BAO_Contact::retrieve($contactParams, $defaults);
-
     $modeFill = $this->isFillDuplicates();
 
-    $groupTree = CRM_Core_BAO_CustomGroup::getTree($params['contact_type'], NULL, $cid, 0, NULL);
-    CRM_Core_BAO_CustomGroup::setDefaults($groupTree, $defaults, FALSE, FALSE);
-
-    $contact = get_object_vars($contactObj);
-
     foreach ($params as $key => $value) {
       if (in_array($key, ['id', 'contact_type'])) {
         continue;
       }
-      // These values must be handled differently because we need to account for location type.
-      $checkLocationType = in_array($key, ['address', 'phone', 'email']);
 
-      if (!$checkLocationType) {
-        if ($customFieldId = CRM_Core_BAO_CustomField::getKeyID($key)) {
-          $custom_params = ['id' => $contact['id'], 'return' => $key];
-          $getValue = civicrm_api3('Contact', 'getvalue', $custom_params);
-          if (empty($getValue)) {
-            unset($getValue);
+      if (!isset($blocks[$key])) {
+        if ($this->isFillDuplicates()) {
+          if (CRM_Core_BAO_CustomField::getKeyID($key)) {
+            $getValue = civicrm_api3('Contact', 'getvalue', ['id' => $cid, 'return' => $key]);
+            if (!empty($getValue)) {
+              // Value exists. Do not overwrite in fill mode.
+              // It is unclear, but historical that we check empty here & NULL below.
+              unset($params[$key]);
+            }
           }
-        }
-        else {
-          $getValue = CRM_Utils_Array::retrieveValueRecursive($contact, $key);
-        }
-
-        if ($modeFill && isset($getValue)) {
-          unset($params[$key]);
-          if ($customFieldId) {
-            // Extra values must be unset to ensure the values are not
-            // imported.
-            unset($params['custom'][$customFieldId]);
+          else {
+            if (($contactObj->$key ?? NULL) !== NULL) {
+              // Value exists. Do not overwrite in fill mode.
+              unset($params[$key]);
+            }
           }
         }
       }
       else {
-
+        // These values must be handled differently because we need to account for location type.
         foreach ($value as $innerKey => $locationValues) {
           if ($modeFill) {
-            $getValue = CRM_Utils_Array::retrieveValueRecursive($contact, $key);
-            if (isset($getValue)) {
-              foreach ($getValue as $cnt => $values) {
-                if ((!empty($getValue[$cnt]['location_type_id']) && !empty($params[$key][$innerKey]['location_type_id'])) && $getValue[$cnt]['location_type_id'] == $params[$key][$innerKey]['location_type_id']) {
-                  unset($params[$key][$innerKey]);
+            foreach ($blocks[$key] ?? [] as $cnt => $values) {
+              if ((!empty($values['location_type_id']) && !empty($params[$key][$innerKey]['location_type_id'])) && $values['location_type_id'] == $params[$key][$innerKey]['location_type_id']) {
+                unset($params[$key][$innerKey]);
+                if (empty($params[$key])) {
+                  unset($params[$key]);
                 }
               }
             }
           }
         }
-        if (count($params[$key]) == 0) {
-          unset($params[$key]);
-        }
       }
     }
   }
@@ -960,7 +481,7 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
   private function getSuccessMessage(): string {
     if (!empty($this->_unparsedStreetAddressContacts)) {
       $errorMessage = ts('Record imported successfully but unable to parse the street address: ');
-      foreach ($this->_unparsedStreetAddressContacts as $contactInfo => $contactValue) {
+      foreach ($this->_unparsedStreetAddressContacts as $contactValue) {
         $contactUrl = CRM_Utils_System::url('civicrm/contact/add', 'reset=1&action=update&cid=' . $contactValue['id'], TRUE, NULL, FALSE);
         $errorMessage .= "\n Contact ID:" . $contactValue['id'] . " <a href=\"$contactUrl\"> " . $contactValue['streetAddress'] . '</a>';
       }
@@ -1013,46 +534,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
     $this->setImportableFieldsMetadata($this->getContactImportMetadata());
   }
 
-  /**
-   * Format contact parameters.
-   *
-   * @todo this function needs re-writing & re-merging into the main function.
-   *
-   * Here be dragons.
-   *
-   * @param array $values
-   * @param array $params
-   *
-   * @return bool
-   */
-  protected function formatContactParameters(&$values, &$params) {
-    // Crawl through the possible classes:
-    // Contact
-    //      Individual
-    //      Household
-    //      Organization
-    //          Location
-    //              Address
-    //              Email
-    //              IM
-    //      Note
-    //      Custom
-
-    // first add core contact values since for other Civi modules they are not added
-    $contactFields = CRM_Contact_DAO_Contact::fields();
-    _civicrm_api3_store_values($contactFields, $values, $params);
-
-    if (isset($values['contact_type'])) {
-      // we're an individual/household/org property
-
-      $fields[$values['contact_type']] = CRM_Contact_DAO_Contact::fields();
-
-      _civicrm_api3_store_values($fields[$values['contact_type']], $values, $params);
-      return TRUE;
-    }
-    return TRUE;
-  }
-
   /**
    * Get metadata for contact importable fields.
    *
@@ -1073,7 +554,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
 
     //CRM-5125
     //supporting import for contact subtypes
-    $csType = NULL;
     if ($this->getContactSubType()) {
       //custom fields for sub type
       $subTypeFields = CRM_Core_BAO_CustomField::getFieldsForImport($this->getContactSubType());
@@ -1105,82 +585,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
     return $fields;
   }
 
-  /**
-   * Format location block ready for importing.
-   *
-   * Note this formatting should all be by the time the code reaches this point
-   *
-   * There is some test coverage for this in
-   * CRM_Contact_Import_Parser_ContactTest e.g. testImportPrimaryAddress.
-   *
-   * @deprecated
-   *
-   * @param array $values
-   *
-   * @return bool
-   * @throws \CRM_Core_Exception
-   */
-  protected function formatLocationBlock(&$values) {
-    // @todo - remove this function.
-    // Original explantion .....
-    // Note: we doing multiple value formatting here for address custom fields, plus putting into right format.
-    // The actual formatting (like date, country ..etc) for address custom fields is taken care of while saving
-    // the address in CRM_Core_BAO_Address::create method
-    if (!empty($values['location_type_id'])) {
-      static $customFields = [];
-      if (empty($customFields)) {
-        $customFields = CRM_Core_BAO_CustomField::getFields('Address');
-      }
-      // make a copy of values, as we going to make changes
-      $newValues = $values;
-      foreach ($values as $key => $val) {
-        $customFieldID = CRM_Core_BAO_CustomField::getKeyID($key);
-        if ($customFieldID && array_key_exists($customFieldID, $customFields)) {
-
-          $htmlType = $customFields[$customFieldID]['html_type'] ?? NULL;
-          if (CRM_Core_BAO_CustomField::isSerialized($customFields[$customFieldID]) && $val) {
-            $mulValues = explode(',', $val);
-            $customOption = CRM_Core_BAO_CustomOption::getCustomOption($customFieldID, TRUE);
-            $newValues[$key] = [];
-            foreach ($mulValues as $v1) {
-              foreach ($customOption as $v2) {
-                if ((strtolower($v2['label']) == strtolower(trim($v1))) ||
-                  (strtolower($v2['value']) == strtolower(trim($v1)))
-                ) {
-                  if ($htmlType == 'CheckBox') {
-                    $newValues[$key][$v2['value']] = 1;
-                  }
-                  else {
-                    $newValues[$key][] = $v2['value'];
-                  }
-                }
-              }
-            }
-          }
-        }
-      }
-      // consider new values
-      $values = $newValues;
-    }
-
-    return TRUE;
-  }
-
-  /**
-   * Get the field metadata for the relevant entity.
-   *
-   * @param string $entity
-   *
-   * @return array
-   */
-  protected function getMetadataForEntity($entity) {
-    if (!isset($this->fieldMetadata[$entity])) {
-      $className = "CRM_Core_DAO_$entity";
-      $this->fieldMetadata[$entity] = $className::fields();
-    }
-    return $this->fieldMetadata[$entity];
-  }
-
   /**
    * Fill in the primary location.
    *
@@ -1376,8 +780,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
    * @param string $prefixString
    *
    * @return array
-   * @throws \CRM_Core_Exception
-   * @throws \Civi\API\Exception\NotImplementedException
    */
   protected function getInvalidValuesForContact($value, string $prefixString): array {
     $errors = [];
@@ -1456,21 +858,6 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
     return $this->getRelationshipType($relationshipTypeID)[$relationshipField];
   }
 
-  /**
-   * Get the related contact type.
-   *
-   * @param int|null $relationshipTypeID
-   * @param int|string $relationshipDirection
-   *
-   * @return null|string
-   *
-   * @throws \CRM_Core_Exception
-   */
-  protected function getRelatedContactLabel($relationshipTypeID, $relationshipDirection): ?string {
-    $relationshipField = 'label_' . $relationshipDirection;
-    return $this->getRelationshipType($relationshipTypeID)[$relationshipField];
-  }
-
   /**
    * Get the relationship type.
    *
@@ -1624,11 +1011,7 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
   protected function processContact(array $params, bool $isMainContact): ?int {
     $contactID = $this->lookupContactID($params, $isMainContact);
     if ($contactID && !empty($params['contact_sub_type'])) {
-      $contactSubType = Contact::get(FALSE)
-        ->addWhere('id', '=', $contactID)
-        ->addSelect('contact_sub_type')
-        ->execute()
-        ->first()['contact_sub_type'];
+      $contactSubType = $this->getExistingContactValue($contactID, 'contact_sub_type');
       if (!empty($contactSubType) && $contactSubType[0] !== $params['contact_sub_type'] && !CRM_Contact_BAO_ContactType::isAllowEdit($contactID, $contactSubType[0])) {
         throw new CRM_Core_Exception('Mismatched contact SubTypes :', CRM_Import_Parser::NO_MATCH);
       }
@@ -1823,4 +1206,21 @@ class CRM_Contact_Import_Parser_Contact extends CRM_Import_Parser {
     return \Civi::$statics[$cacheString];
   }
 
+  /**
+   * Get the metadata field for which importable fields does not key the actual field name.
+   *
+   * @return string[]
+   */
+  protected function getOddlyMappedMetadataFields(): array {
+    return [
+      'country_id' => 'country',
+      'state_province_id' => 'state_province',
+      'county_id' => 'county',
+      'email_greeting_id' => 'email_greeting',
+      'postal_greeting_id' => 'postal_greeting',
+      'addressee_id' => 'addressee',
+      'source' => 'contact_source',
+    ];
+  }
+
 }
