@@ -212,7 +212,7 @@ class CRM_Contact_Selector extends CRM_Core_Selector_Base implements CRM_Core_Se
     }
 
     $displayRelationshipType = $this->_formValues['display_relationship_type'] ?? NULL;
-    $operator = CRM_Utils_Array::value('operator', $this->_formValues, 'AND');
+    $operator = $this->_formValues['operator'] ?? 'AND';
 
     // rectify params to what proximity search expects if there is a value for prox_distance
     // CRM-7021
@@ -308,6 +308,10 @@ class CRM_Contact_Selector extends CRM_Core_Selector_Base implements CRM_Core_Se
             $url = 'civicrm/contact/view/activity';
             $qs = "atype=3&action=add&reset=1&cid=%%id%%{$extraParams}";
           }
+          elseif ($value['key'] === 'note') {
+            $url = 'civicrm/note';
+            $qs = "reset=1&action=add&entity_table=civicrm_contact&entity_id='%%id%%{$extraParams}";
+          }
 
           self::$_links[$counter++] = [
             'name' => $value['title'],
@@ -421,7 +425,7 @@ class CRM_Contact_Selector extends CRM_Core_Selector_Base implements CRM_Core_Se
           if (!empty($field['in_selector']) &&
             !in_array($name, $skipFields)
           ) {
-            if (strpos($name, '-') !== FALSE) {
+            if (str_contains($name, '-')) {
               [$fieldName, $lType, $type] = CRM_Utils_System::explode('-', $name, 3);
 
               if ($lType === 'Primary') {
@@ -610,7 +614,7 @@ class CRM_Contact_Selector extends CRM_Core_Selector_Base implements CRM_Core_Se
         if (!empty($field['in_selector']) &&
           !in_array($key, $skipFields)
         ) {
-          if (strpos($key, '-') !== FALSE) {
+          if (str_contains($key, '-')) {
             [$fieldName, $id, $type] = CRM_Utils_System::explode('-', $key, 3);
 
             if ($id === 'Primary') {
@@ -705,12 +709,9 @@ class CRM_Contact_Selector extends CRM_Core_Selector_Base implements CRM_Core_Se
           $row[$property] = $result->$greeting;
         }
         elseif (isset($pseudoconstants[$property])) {
-          $row[$property] = CRM_Utils_Array::value(
-            $result->{$pseudoconstants[$property]['dbName']},
-            $pseudoconstants[$property]['values']
-          );
+          $row[$property] = $pseudoconstants[$property]['values'][$result->{$pseudoconstants[$property]['dbName']}] ?? NULL;
         }
-        elseif (strpos($property, '-url') !== FALSE) {
+        elseif (str_contains($property, '-url')) {
           $websiteUrl = '';
           $websiteKey = str_replace('-url', '', $property);
           $propertyArray = explode('-', $property);
@@ -723,7 +724,7 @@ class CRM_Contact_Selector extends CRM_Core_Selector_Base implements CRM_Core_Se
           }
           $row[$property] = $websiteUrl;
         }
-        elseif (strpos($property, '-email') !== FALSE) {
+        elseif (str_contains($property, '-email')) {
           [$locType] = explode("-email", $property);
           $onholdProperty = "{$locType}-on_hold";
 
