@@ -124,7 +124,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
 
     // Make the contributionPageID available to the template
     $this->assign('contributionPageID', $this->_id);
-    $this->assign('ccid', $this->_ccid);
+    $this->assign('ccid', $this->getExistingContributionID());
     $this->assign('isShare', $this->_values['is_share'] ?? NULL);
     $this->assign('isConfirmEnabled', $this->_values['is_confirm_enabled'] ?? NULL);
 
@@ -318,7 +318,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
   public function buildQuickForm() {
     // build profiles first so that we can determine address fields etc
     // and then show copy address checkbox
-    if (empty($this->_ccid)) {
+    if (empty($this->getExistingContributionID())) {
       $this->buildCustom($this->_values['custom_pre_id'], 'customPre');
       $this->buildCustom($this->_values['custom_post_id'], 'customPost');
 
@@ -340,8 +340,8 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
     }
 
     $this->applyFilter('__ALL__', 'trim');
-    $this->assign('showMainEmail', (empty($this->_ccid) && $this->_emailExists === FALSE));
-    if (empty($this->_ccid)) {
+    $this->assign('showMainEmail', (empty($this->getExistingContributionID()) && $this->_emailExists === FALSE));
+    if (empty($this->getExistingContributionID())) {
       if ($this->_emailExists == FALSE) {
         $this->add('text', "email-{$this->_bltID}",
           ts('Email Address'),
@@ -366,7 +366,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
 
     //build pledge block.
     //don't build membership block when pledge_id is passed
-    if (empty($this->_values['pledge_id']) && empty($this->_ccid)) {
+    if (empty($this->_values['pledge_id']) && empty($this->getExistingContributionID())) {
       $this->_separateMembershipPayment = FALSE;
       if (CRM_Core_Component::isEnabled('CiviMember')) {
         $this->_separateMembershipPayment = $this->buildMembershipBlock();
@@ -382,8 +382,8 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
       $this->add('hidden', 'priceSetId', $this->_priceSetId);
       // build price set form.
       $this->set('priceSetId', $this->_priceSetId);
-      if (empty($this->_ccid)) {
-        $this->buildPriceSet($this);
+      if (empty($this->getExistingContributionID())) {
+        $this->buildPriceSet();
       }
       if ($this->_values['is_monetary'] &&
         $this->_values['is_recur'] && empty($this->_values['pledge_id'])
@@ -393,19 +393,19 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
     }
 
     //we allow premium for pledge during pledge creation only.
-    if (empty($this->_values['pledge_id']) && empty($this->_ccid)) {
+    if (empty($this->_values['pledge_id']) && empty($this->getExistingContributionID())) {
       $this->buildPremiumsBlock(TRUE);
     }
 
     //don't build pledge block when mid is passed
-    if (!$this->getRenewalMembershipID() && empty($this->_ccid)) {
+    if (!$this->getRenewalMembershipID() && empty($this->getExistingContributionID())) {
       if (CRM_Core_Component::isEnabled('CiviPledge') && !empty($this->_values['pledge_block_id'])) {
         $this->buildPledgeBlock();
       }
     }
 
     //to create an cms user
-    if (!$this->_contactID && empty($this->_ccid)) {
+    if (!$this->_contactID && empty($this->getExistingContributionID())) {
       $createCMSUser = FALSE;
 
       if ($this->_values['custom_pre_id']) {
@@ -435,7 +435,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
         CRM_Core_BAO_CMSUser::buildForm($this, $profileID, TRUE);
       }
     }
-    if ($this->getPcpID() && empty($this->_ccid)) {
+    if ($this->getPcpID() && empty($this->getExistingContributionID())) {
       if (CRM_PCP_BAO_PCP::displayName($this->_pcpId)) {
         $pcp_supporter_text = CRM_PCP_BAO_PCP::getPcpSupporterText($this->_pcpId, $this->_id, 'contribute');
       }
@@ -454,7 +454,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
       }
     }
     $this->assign('pcpSupporterText', $pcp_supporter_text ?? NULL);
-    if (empty($this->_values['fee']) && empty($this->_ccid)) {
+    if (empty($this->_values['fee']) && empty($this->getExistingContributionID())) {
       throw new CRM_Core_Exception(ts('This page does not have any price fields configured or you may not have permission for them. Please contact the site administrator for more details.'));
     }
 
@@ -493,7 +493,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
         $this->submitOnce = TRUE;
       }
       //change button name for updating contribution
-      if (!empty($this->_ccid)) {
+      if (!empty($this->getExistingContributionID())) {
         $submitButton['name'] = ts('Confirm Payment');
       }
       $this->addButtons([$submitButton]);
@@ -505,20 +505,18 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
   /**
    * Build the price set form.
    *
-   * @param CRM_Core_Form $form
-   *
    * @return void
    * @throws \CRM_Core_Exception
    */
-  private function buildPriceSet($form) {
+  private function buildPriceSet() {
     $validPriceFieldIds = array_keys($this->getPriceFieldMetaData());
-    $form->assign('priceSet', $form->_priceSet);
+    $this->assign('priceSet', $this->_priceSet);
     $this->assign('membershipFieldID');
 
     // @todo - this hook wrangling can be done earlier if we set the form on $this->>order.
-    $feeBlock = &$form->_values['fee'];
+    $feeBlock = &$this->_values['fee'];
     // Call the buildAmount hook.
-    CRM_Utils_Hook::buildAmount($this->getFormContext(), $form, $feeBlock);
+    CRM_Utils_Hook::buildAmount($this->getFormContext(), $this, $feeBlock);
 
     // CRM-14492 Admin price fields should show up on event registration if user has 'administer CiviCRM' permissions
     $adminFieldVisible = CRM_Core_Permission::check('administer CiviCRM');
@@ -573,7 +571,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
             }
           }
 
-          CRM_Price_BAO_PriceField::addQuickFormElement($form,
+          CRM_Price_BAO_PriceField::addQuickFormElement($this,
             'price_' . $fieldID,
             $field['id'],
             FALSE,
@@ -586,7 +584,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
         }
       }
     }
-    $form->assign('hasExistingLifetimeMembership', $checklifetime);
+    $this->assign('hasExistingLifetimeMembership', $checklifetime);
   }
 
   /**
@@ -992,23 +990,17 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
           $errors['_qf_default'] = ts('Please select at least one membership option.');
         }
       }
-      // @todo - processAmount is to be deprectated - can we use getTotalAmount or
-      // a function of self->order here?
-      CRM_Price_BAO_PriceSet::processAmount($self->_values['fee'],
-        $fields
-      );
 
+      $amount = $self->getOrder()->getTotalAmount();
       $minAmt = CRM_Core_DAO::getFieldValue('CRM_Price_DAO_PriceSet', $fields['priceSetId'], 'min_amount');
-      if ($fields['amount'] < 0) {
+      if ($amount < 0) {
         $errors['_qf_default'] = ts('Contribution can not be less than zero. Please select the options accordingly');
       }
-      elseif (!empty($minAmt) && $fields['amount'] < $minAmt) {
+      elseif (!empty($minAmt) && $amount < $minAmt) {
         $errors['_qf_default'] = ts('A minimum amount of %1 should be selected from Contribution(s).', [
           1 => CRM_Utils_Money::format($minAmt),
         ]);
       }
-
-      $amount = $fields['amount'];
     }
 
     if (isset($fields['selectProduct']) &&
@@ -1225,7 +1217,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
 
     $params['amount'] = $this->getMainContributionAmount();
     $this->set('amount_level', $this->order->getAmountLevel());
-    if (!empty($this->_ccid)) {
+    if (!empty($this->getExistingContributionID())) {
       // @todo - verify that this is the same as `$this->>getLineItems()` which it should be & consolidate
       $this->set('lineItem', [$this->getPriceSetID() => $this->getExistingContributionLineItems()]);
     }
@@ -1252,7 +1244,7 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
     $this->set('amount', $this->getMainContributionAmount());
 
     // generate and set an invoiceID for this transaction
-    $invoiceID = md5(uniqid(rand(), TRUE));
+    $invoiceID = bin2hex(random_bytes(16));
     $this->set('invoiceID', $invoiceID);
     $params['invoiceID'] = $invoiceID;
     $title = !empty($this->_values['frontend_title']) ? $this->_values['frontend_title'] : $this->_values['title'];
@@ -1338,10 +1330,9 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
     if (empty($this->getExistingContributionID())) {
       return;
     }
-    // @todo - all this stuff is likely obsolete.
-    if ($taxAmount = CRM_Core_DAO::getFieldValue('CRM_Contribute_DAO_Contribution', $this->_ccid, 'tax_amount')) {
-      $this->assign('taxAmount', $taxAmount);
-    }
+
+    $this->assign('taxAmount', $this->getContributionValue('tax_amount'));
+    $this->assign('taxTerm', Civi::settings()->get('tax_term'));
 
     $lineItems = $this->getExistingContributionLineItems();
     $this->assign('lineItem', [$this->getPriceSetID() => $lineItems]);
@@ -1363,8 +1354,11 @@ class CRM_Contribute_Form_Contribution_Main extends CRM_Contribute_Form_Contribu
     if (!$this->getContactID()) {
       CRM_Core_Error::statusBounce(ts('Returning since there is no contact attached to this contribution id.'));
     }
+    if ($this->getContributionValue('contribution_status_id:name') === 'Cancelled') {
+      throw new CRM_Core_Exception(ts('Sorry, this contribution has been cancelled.'));
+    }
 
-    $paymentBalance = CRM_Contribute_BAO_Contribution::getContributionBalance($this->_ccid);
+    $paymentBalance = CRM_Contribute_BAO_Contribution::getContributionBalance($this->getExistingContributionID());
     //bounce if the contribution is not pending.
     if ((float) $paymentBalance <= 0) {
       CRM_Core_Error::statusBounce(ts('Returning since contribution has already been handled.'));
