@@ -181,7 +181,7 @@ class CRM_Contribute_Form_ContributionView extends CRM_Core_Form {
     }
 
     // Get Note
-    $noteValue = CRM_Core_BAO_Note::getNote(CRM_Utils_Array::value('id', $values), 'civicrm_contribution');
+    $noteValue = CRM_Core_BAO_Note::getNote($values['id'], 'civicrm_contribution');
     $values['note'] = array_values($noteValue);
 
     // show billing address location details, if exists
