@@ -32,11 +32,7 @@ class CRM_Core_BAO_CustomValueTable {
     ) {
       return;
     }
-    CRM_Utils_Hook::customParamsPre($hookOP,
-      $hookID,
-      $entityID,
-      $customParams
-    );
+
     $paramFieldsExtendContactForEntities = [];
     $VS = CRM_Core_DAO::VALUE_SEPARATOR;
 
@@ -73,107 +69,33 @@ class CRM_Core_BAO_CustomValueTable {
 
         foreach ($fields as $field) {
           // fix the value before we store it
-          $value = $field['value'];
+          $serialize = $field['serialize'] ?? NULL;
+          $value = $serialize ? CRM_Core_DAO::serializeField($field['value'], $serialize) : $field['value'];
           $type = $field['type'];
+
           switch ($type) {
             case 'StateProvince':
-              $type = 'Integer';
-              if (is_array($value)) {
-                $value = CRM_Core_DAO::VALUE_SEPARATOR . implode(CRM_Core_DAO::VALUE_SEPARATOR, $value) . CRM_Core_DAO::VALUE_SEPARATOR;
-                $type = 'String';
-              }
-              elseif (!is_numeric($value) && !strstr($value, CRM_Core_DAO::VALUE_SEPARATOR)) {
-                //fix for multi select state, CRM-3437
-                $mulValues = explode(',', $value);
-                $validStates = [];
-                foreach ($mulValues as $key => $stateVal) {
-                  $states = [];
-                  $states['state_province'] = trim($stateVal);
-
-                  CRM_Utils_Array::lookupValue($states, 'state_province',
-                    CRM_Core_PseudoConstant::stateProvince(), TRUE
-                  );
-                  if (empty($states['state_province_id'])) {
-                    CRM_Utils_Array::lookupValue($states, 'state_province',
-                      CRM_Core_PseudoConstant::stateProvinceAbbreviation(), TRUE
-                    );
-                  }
-                  $validStates[] = $states['state_province_id'] ?? NULL;
-                }
-                $value = implode(CRM_Core_DAO::VALUE_SEPARATOR,
-                  $validStates
-                );
-                $type = 'String';
-              }
-              elseif (!$value) {
-                // CRM-3415
-                // using type of timestamp allows us to sneak in a null into db
-                // gross but effective hack
-                $value = NULL;
-                $type = 'Timestamp';
-              }
-              else {
-                $type = 'String';
-              }
-              break;
-
             case 'Country':
-              $type = 'Integer';
-              if (is_array($value)) {
-                $value = CRM_Core_DAO::VALUE_SEPARATOR . implode(CRM_Core_DAO::VALUE_SEPARATOR, $value) . CRM_Core_DAO::VALUE_SEPARATOR;
-                $type = 'String';
-              }
-              elseif (!is_numeric($value) && !strstr($value, CRM_Core_DAO::VALUE_SEPARATOR)) {
-                //fix for multi select country, CRM-3437
-                $mulValues = explode(',', $value);
-                $validCountries = [];
-                foreach ($mulValues as $key => $countryVal) {
-                  $countries = [];
-                  $countries['country'] = trim($countryVal);
-                  CRM_Utils_Array::lookupValue($countries, 'country',
-                    CRM_Core_PseudoConstant::country(), TRUE
-                  );
-                  if (empty($countries['country_id'])) {
-                    CRM_Utils_Array::lookupValue($countries, 'country',
-                      CRM_Core_PseudoConstant::countryIsoCode(), TRUE
-                    );
-                  }
-                  $validCountries[] = $countries['country_id'] ?? NULL;
-                }
-                $value = implode(CRM_Core_DAO::VALUE_SEPARATOR,
-                  $validCountries
-                );
-                $type = 'String';
-              }
-              elseif (!$value) {
+              $type = $serialize ? 'String' : 'Integer';
+              if (!$value) {
                 // CRM-3415
                 // using type of timestamp allows us to sneak in a null into db
                 // gross but effective hack
                 $value = NULL;
                 $type = 'Timestamp';
               }
-              else {
-                $type = 'String';
-              }
               break;
 
             case 'File':
+              if (!empty($field['id'])) {
+                self::deleteFile($field);
+              }
               if (!$field['file_id']) {
                 $value = 'null';
                 break;
               }
-
-              // need to add/update civicrm_entity_file
-              $entityFileDAO = new CRM_Core_DAO_EntityFile();
-              $entityFileDAO->file_id = $field['file_id'];
-              $entityFileDAO->find(TRUE);
-
-              $entityFileDAO->entity_table = $field['table_name'];
-              $entityFileDAO->entity_id = $field['entity_id'];
-              $entityFileDAO->file_id = $field['file_id'];
-              $entityFileDAO->save();
               $value = $field['file_id'];
-              $type = 'String';
+              $type = 'Integer';
               break;
 
             case 'Date':
@@ -190,26 +112,32 @@ class CRM_Core_BAO_CustomValueTable {
               break;
 
             case 'ContactReference':
-              if ($value == NULL || $value === '' || $value === $VS . $VS) {
-                $type = 'Timestamp';
-                $value = NULL;
-              }
-              elseif (strpos($value, $VS) !== FALSE) {
+              if ($serialize) {
                 $type = 'String';
                 // Validate the string contains only integers and value-separators
                 $validChars = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, $VS];
                 if (str_replace($validChars, '', $value)) {
                   throw new CRM_Core_Exception('Contact ID must be of type Integer');
                 }
+                // Prevent saving an empty "array" which results in a fatal error on render.
+                if ($value === '' || $value === $VS . $VS) {
+                  $value = NULL;
+                }
               }
               else {
                 $type = 'Integer';
               }
+              // An empty value should be stored as NULL
+              if (!$value) {
+                $type = 'Timestamp';
+                $value = NULL;
+              }
               break;
 
             case 'EntityReference':
               $type = 'Integer';
-              if ($value == NULL || $value === '') {
+              // An empty value should be stored as NULL
+              if (!$value) {
                 $type = 'Timestamp';
                 $value = NULL;
               }
@@ -300,26 +228,25 @@ class CRM_Core_BAO_CustomValueTable {
    * Given a field return the mysql data type associated with it.
    *
    * @param string $type
-   * @param int $maxLength
+   * @param int|null $maxLength
    *
    * @return string
    *   the mysql data store placeholder
    */
-  public static function fieldToSQLType($type, $maxLength = 255) {
-    if (!isset($maxLength) ||
-      !is_numeric($maxLength) ||
-      $maxLength <= 0
-    ) {
-      $maxLength = 255;
-    }
-
+  public static function fieldToSQLType(string $type, $maxLength = NULL) {
     switch ($type) {
       case 'String':
+        $maxLength = $maxLength ?: 255;
+        return "varchar($maxLength)";
+
       case 'Link':
+        // URLs can be up to 2047 characters
+        // according to https://www.sitemaps.org/protocol.html#locdef
+        $maxLength = $maxLength ?: 2047;
         return "varchar($maxLength)";
 
       case 'Boolean':
-        return 'tinyint';
+        return 'boolean';
 
       case 'Int':
         return 'int';
@@ -506,17 +433,12 @@ AND    $cond
     $dao = CRM_Core_DAO::executeQuery($query);
 
     $select = $fields = $isMultiple = [];
-    $dateField = '';
 
     while ($dao->fetch()) {
       if (!array_key_exists($dao->table_name, $select)) {
         $fields[$dao->table_name] = [];
         $select[$dao->table_name] = [];
       }
-      if ($dao->fieldDataType == 'Date') {
-        $dateField = $dao->column_name;
-      }
-
       $fields[$dao->table_name][] = $dao->fieldID;
       $select[$dao->table_name][] = "{$dao->column_name} AS custom_{$dao->fieldID}";
       $isMultiple[$dao->table_name] = (bool) $dao->is_multiple;
@@ -534,10 +456,6 @@ AND    $cond
         }
       }
 
-      //by default sort by date field desc
-      if (empty($orderBy) && $formatMultiRecordField && $dateField) {
-        $orderBy = ' ORDER BY ' . CRM_Utils_Type::escape($dateField . ' DESC ', 'String');
-      }
       $query = "SELECT SQL_CALC_FOUND_ROWS id, " . implode(', ', $clauses) . " FROM $tableName WHERE entity_id = $entityID {$orderBy} {$limit}";
       $dao = CRM_Core_DAO::executeQuery($query);
       if (!empty($DTparams)) {
@@ -786,4 +704,23 @@ AND    $cond
     }
   }
 
+  /**
+   * Delete orphaned files from disk when updating custom file fields
+   */
+  private static function deleteFile(array $field) {
+    $sql = CRM_Utils_SQL_Select::from($field['table_name'])
+      ->select($field['column_name'])
+      ->where("id = #id", ['#id' => $field['id']])
+      ->toSQL();
+    $fileId = CRM_Core_DAO::singleValueQuery($sql);
+    if ($fileId && $fileId != ($field['file_id'] ?? NULL)) {
+      $refCount = \Civi\Api4\Utils\CoreUtil::getRefCountTotal('File', $fileId);
+      if ($refCount <= 1) {
+        \Civi\Api4\File::delete(FALSE)
+          ->addWhere('id', '=', $fileId)
+          ->execute();
+      }
+    }
+  }
+
 }
