@@ -144,12 +144,6 @@ class CRM_Core_BAO_OptionValue extends CRM_Core_DAO_OptionValue implements \Civi
     $optionValue = new CRM_Core_DAO_OptionValue();
     $optionValue->copyValues($params);
 
-    $isDomainOptionGroup = CRM_Core_OptionGroup::isDomainOptionGroup($groupName);
-    // When creating a new option for a group that requires a domain, set default domain
-    if ($isDomainOptionGroup && empty($params['id']) && (empty($params['domain_id']) || CRM_Utils_System::isNull($params['domain_id']))) {
-      $optionValue->domain_id = CRM_Core_Config::domainID();
-    }
-
     $groupsSupportingDuplicateValues = ['languages'];
     if (!$id && !empty($params['value'])) {
       $dao = new CRM_Core_DAO_OptionValue();
@@ -160,9 +154,6 @@ class CRM_Core_BAO_OptionValue extends CRM_Core_DAO_OptionValue implements \Civi
         // CRM-21737 languages option group does not use unique values but unique names.
         $dao->name = $params['name'];
       }
-      if (CRM_Core_OptionGroup::isDomainOptionGroup($groupName)) {
-        $dao->domain_id = $optionValue->domain_id;
-      }
       $dao->option_group_id = $params['option_group_id'];
       if ($dao->find(TRUE)) {
         throw new CRM_Core_Exception('Value already exists in the database');
