@@ -237,7 +237,7 @@ class CRM_Core_EntityTokens extends AbstractTokenSubscriber {
    * @return bool
    */
   protected function isPseudoField(string $fieldName): bool {
-    return strpos($fieldName, ':') !== FALSE;
+    return str_contains($fieldName, ':');
   }
 
   /**
@@ -440,7 +440,7 @@ class CRM_Core_EntityTokens extends AbstractTokenSubscriber {
     // 'not a real field' offered up by case - seems like an oddity
     // we should skip at the top level for now.
     $fields = ['tags'];
-    if (!CRM_Campaign_BAO_Campaign::isComponentEnabled()) {
+    if (!CRM_Core_Component::isEnabled('CiviCampaign')) {
       $fields[] = 'campaign_id';
     }
     return $fields;
@@ -661,7 +661,7 @@ class CRM_Core_EntityTokens extends AbstractTokenSubscriber {
       // the space open for that.
       // Not the existing QuickForm widget has handling for the custom field
       // format based on the title using this syntax.
-      $parts = explode(': ', $field['label']);
+      $parts = explode(': ', $field['label'], 2);
       $field['title'] = "{$parts[1]} :: {$parts[0]}";
       $tokenName = 'custom_' . $field['custom_field_id'];
       $tokensMetadata[$tokenName] = $field;
