@@ -131,8 +131,7 @@ class CRM_Core_Page_AJAX_Location {
         }
         $elements["onbehalf_{$field}-{$locTypeId}"] = [
           'type' => $type,
-          'value' => isset($location['address'][1]) ? CRM_Utils_Array::value($addField,
-            $location['address'][1]) : NULL,
+          'value' => $location['address'][1][$addField] ?? NULL,
         ];
         unset($profileFields["{$field}-{$locTypeId}"]);
       }
