@@ -457,7 +457,7 @@ class CRM_Core_Payment_AuthorizeNet extends CRM_Core_Payment {
    * @return bool|string
    */
   public function _substring_between(&$haystack, $start, $end) {
-    if (strpos($haystack, $start) === FALSE || strpos($haystack, $end) === FALSE) {
+    if (!str_contains($haystack, $start) || !str_contains($haystack, $end)) {
       return FALSE;
     }
     else {
@@ -479,7 +479,7 @@ class CRM_Core_Payment_AuthorizeNet extends CRM_Core_Payment {
    *   not set
    */
   public function _getParam($field, $xmlSafe = FALSE) {
-    $value = CRM_Utils_Array::value($field, $this->_params, '');
+    $value = $this->_params[$field] ?? '';
     if ($xmlSafe) {
       $value = str_replace(['&', '"', "'", '<', '>'], '', $value);
     }
