@@ -207,7 +207,7 @@ class CRM_Core_Payment_PayPalProIPN {
    */
   public function retrieve($name, $type, $abort = TRUE) {
     $value = CRM_Utils_Type::validate(
-      CRM_Utils_Array::value($name, $this->_inputParameters),
+      $this->_inputParameters[$name] ?? NULL,
       $type,
       FALSE
     );
@@ -261,7 +261,7 @@ class CRM_Core_Payment_PayPalProIPN {
           // In future moving to create pending & then complete, but this OK for now.
           // Also consider accepting 'Failed' like other processors.
           $input['contribution_status_id'] = CRM_Core_PseudoConstant::getKey('CRM_Contribute_BAO_Contribution', 'contribution_status_id', 'Completed');
-          $input['invoice_id'] = md5(uniqid(rand(), TRUE));
+          $input['invoice_id'] = bin2hex(random_bytes(16));
           $input['original_contribution_id'] = $this->getContributionID();
           $input['contribution_recur_id'] = $this->getContributionRecurID();
 
