@@ -572,7 +572,7 @@ class CRM_Core_Permission {
       $item['access_callback'][0] == 'CRM_Core_Permission' &&
       $item['access_callback'][1] == 'checkMenu'
     ) {
-      $op = CRM_Utils_Array::value(1, $item['access_arguments'], 'and');
+      $op = $item['access_arguments'][1] ?? 'and';
       return self::checkMenu($item['access_arguments'][0], $op);
     }
     else {
@@ -757,7 +757,7 @@ class CRM_Core_Permission {
       ],
       'profile listings' => [
         'label' => $prefix . ts('profile listings'),
-        'description' => ts('Warning: Give to trusted roles only; this permission has privacy implications. Access public searchable directories.'),
+        'description' => ts('Warning: Give to trusted roles only; this permission has privacy implications. Access public searchable directories.') . ' ' . ts('Profile listings are being phased out in favor of SearchKit/FormBuilder. They have been moved to the Legacy Profiles extension.'),
       ],
       'profile create' => [
         'label' => $prefix . ts('profile create'),
@@ -1084,6 +1084,9 @@ class CRM_Core_Permission {
       ],
       // managed by query object
       'get' => [],
+      'getMergedTo' => [],
+      'getMergedFrom' => [],
+
       // managed by _civicrm_api3_check_edit_permissions
       'update' => [],
       'duplicatecheck' => [
@@ -1825,7 +1828,7 @@ class CRM_Core_Permission {
    * @return bool
    */
   public static function isMultisiteEnabled() {
-    return (bool) Civi::settings()->get('is_enabled');
+    return (bool) Civi::settings()->get('multisite_is_enabled');
   }
 
   /**
