@@ -161,24 +161,6 @@ class CRM_Core_Smarty extends CRM_Core_SmartyCompatibility {
       $this->assign('langSwitch', CRM_Core_I18n::uiLanguages());
     }
 
-    if (CRM_Utils_Constant::value('CIVICRM_SMARTY_DEFAULT_ESCAPE')
-      && !CRM_Utils_Constant::value('CIVICRM_SMARTY3_AUTOLOAD_PATH')
-      && !CRM_Utils_Constant::value('CIVICRM_SMARTY_AUTOLOAD_PATH')
-    ) {
-      // Currently DEFAULT escape does not work with Smarty3
-      // dunno why - thought it would be the default with Smarty3 - but
-      // getting onto Smarty 3 is higher priority.
-      // The include below loads the v2 version which is why id doesn't work.
-      // When default escape is enabled if the core escape is called before
-      // any custom escaping is done the modifier_escape function is not
-      // found, so require_once straight away. Note this was hit on the basic
-      // contribution dashboard from RecentlyViewed.tpl
-      require_once 'Smarty/plugins/modifier.escape.php';
-      if (!isset($this->_plugins['modifier']['escape'])) {
-        $this->registerPlugin('modifier', 'escape', ['CRM_Core_Smarty', 'escape']);
-      }
-      $this->default_modifiers[] = 'escape:"htmlall"';
-    }
     $this->loadFilter('pre', 'resetExtScope');
     $this->loadFilter('pre', 'htxtFilter');
 
@@ -298,23 +280,42 @@ class CRM_Core_Smarty extends CRM_Core_SmartyCompatibility {
   }
 
   /**
-   * Avoid e-notices on pages with tabs,
-   * by ensuring tabHeader items contain the necessary keys
+   * @deprecated
+   * Directly apply self::setRequiredTemplateTabKeys to the tabHeader
+   * variable
    */
   public function addExpectedTabHeaderKeys(): void {
+    $tabs = $this->getTemplateVars('tabHeader');
+    $tabs = self::setRequiredTabTemplateKeys($tabs);
+    $this->assign('tabHeader', $tabs);
+  }
+
+  /**
+   * Ensure an array of tabs has the required keys to be passed
+   * to our Smarty tabs templates (TabHeader.tpl or Summary.tpl)
+   */
+  public static function setRequiredTabTemplateKeys(array $tabs): array {
     $defaults = [
       'class' => '',
       'extra' => '',
-      'icon' => FALSE,
-      'count' => FALSE,
-      'template' => FALSE,
+      'icon' => NULL,
+      'count' => NULL,
+      'hideCount' => FALSE,
+      'template' => NULL,
+      'active' => TRUE,
+      'valid' => TRUE,
+      // Afform tabs set the afform module and directive - NULL for non-afform tabs
+      'module' => NULL,
+      'directive' => NULL,
     ];
 
-    $tabs = $this->getTemplateVars('tabHeader');
-    foreach ((array) $tabs as $i => $tab) {
-      $tabs[$i] = array_merge($defaults, $tab);
+    foreach ($tabs as $i => $tab) {
+      if (empty($tab['url'])) {
+        $tab['url'] = $tab['link'] ?? '';
+      }
+      $tabs[$i] = array_merge($defaults, (array) $tab);
     }
-    $this->assign('tabHeader', $tabs);
+    return $tabs;
   }
 
   /**
@@ -350,7 +351,7 @@ class CRM_Core_Smarty extends CRM_Core_SmartyCompatibility {
       $this->assign($name, $value);
     }
     else {
-      if (strpos($currentValue, $value) === FALSE) {
+      if (!str_contains($currentValue, $value)) {
         $this->assign($name, $currentValue . $value);
       }
     }
@@ -485,7 +486,7 @@ class CRM_Core_Smarty extends CRM_Core_SmartyCompatibility {
    * anything coming in with this be happening because of the default modifier.
    *
    * Also note the right way to opt a field OUT of escaping is
-   * ``{$fieldName|smarty:nodefaults}``
+   * ``{$fieldName nofilter}``
    * This should be used for fields with known html AND for fields where
    * we are doing empty or isset checks - as otherwise the value is passed for
    * escaping first so you still get an enotice for 'empty' or a fatal for 'isset'
@@ -514,37 +515,38 @@ class CRM_Core_Smarty extends CRM_Core_SmartyCompatibility {
     if ($esc_type === 'htmlall') {
       // 'htmlall' is the nothing-specified default.
       // Don't escape things we think quickform added.
-      if (strpos($string, '<input') === 0
-        || strpos($string, '<select') === 0
+      if (str_starts_with($string, '<input')
+        || str_starts_with($string, '<select')
         // Not handling as yet but these ones really should get some love.
-        || strpos($string, '<label') === 0
-        || strpos($string, '<button') === 0
-        || strpos($string, '<span class="crm-frozen-field">') === 0
-        || strpos($string, '<textarea') === 0
+        || str_starts_with($string, '<label')
+        || str_starts_with($string, '<button')
+        || str_starts_with($string, '<span class="crm-frozen-field">')
+        || str_starts_with($string, '<textarea')
 
         // The ones below this point are hopefully here short term.
-        || strpos($string, '<a') === 0
+        || str_starts_with($string, '<a')
         // Message templates screen
-        || strpos($string, '<span><a href') === 0
+        || str_starts_with($string, '<span><a href')
         // Not sure how big a pattern this is - used in Pledge view tab
         // not sure if it needs escaping
-        || strpos($string, ' action="/civicrm/') === 0
+        || str_starts_with($string, ' action="/civicrm/')
         // eg. Tag edit page, civicrm/admin/financial/financialType/accounts?action=add&reset=1&aid=1
-        || strpos($string, ' action="" method="post"') === 0
+        || str_starts_with($string, ' action="" method="post"')
         // This seems to be urls...
-        || strpos($string, '/civicrm/') === 0
+        || str_starts_with($string, '/civicrm/')
         // Validation error message - eg. <span class="crm-error">Tournament Fees is a required field.</span>
         || strpos($string, '
     <span class="crm-error">') === 0
         // e.g from participant tab class="action-item" href=/civicrm/contact/view/participant?reset=1&amp;action=add&amp;cid=142&amp;context=participant
-        || strpos($string, 'class="action-item" href=/civicrm/"') === 0
+        || str_starts_with($string, 'class="action-item" href=/civicrm/"')
       ) {
         // Do not escape the above common patterns.
         return $string;
       }
     }
 
-    $value = smarty_modifier_escape($string, $esc_type, $char_set);
+    $string = mb_convert_encoding($string, 'UTF-8', $char_set);
+    $value = htmlentities($string, ENT_QUOTES, 'UTF-8');
     if ($value !== $string) {
       Civi::log('smarty')->debug('smarty escaping original {original}, escaped {escaped} type {type} charset {charset}', [
         'original' => $string,
@@ -557,6 +559,10 @@ class CRM_Core_Smarty extends CRM_Core_SmartyCompatibility {
   }
 
   public function getVersion (): int {
+    return static::findVersion();
+  }
+
+  public static function findVersion(): int {
     static $version;
     if ($version === NULL) {
       if (class_exists('Smarty\Smarty')) {
