@@ -62,15 +62,6 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
    */
   public $_feeBlock;
 
-  /**
-   * Array of payment related fields to potentially display on this form (generally credit card or debit card fields).
-   *
-   * This is rendered via billingBlock.tpl.
-   *
-   * @var array
-   */
-  public $_paymentFields = [];
-
   /**
    * Is this submission incurring no costs.
    *
@@ -163,7 +154,7 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
     //here we can't use parent $this->_allowWaitlist as user might
     //walk back and we might set this value in this postProcess.
     //(we set when spaces < group count and want to allow become part of waiting )
-    $eventFull = CRM_Event_BAO_Participant::eventFull($this->_eventId, FALSE, CRM_Utils_Array::value('has_waitlist', $this->_values['event']));
+    $eventFull = CRM_Event_BAO_Participant::eventFull($this->_eventId, FALSE, $this->_values['event']['has_waitlist'] ?? NULL);
 
     // Get payment processors if appropriate for this event
     $this->_noFees = $suppressPayment = $this->isSuppressPayment();
@@ -250,9 +241,7 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
     if (!empty($this->_fields)) {
       //load default campaign from page.
       if (array_key_exists('participant_campaign_id', $this->_fields)) {
-        $this->_defaults['participant_campaign_id'] = CRM_Utils_Array::value('campaign_id',
-          $this->_values['event']
-        );
+        $this->_defaults['participant_campaign_id'] = $this->_values['event']['campaign_id'] ?? NULL;
       }
 
       foreach ($this->_fields as $name => $field) {
@@ -410,9 +399,8 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
       $this->_waitlistMsg = ts("This event has only %1 space(s) left. If you continue and register more than %1 people (including yourself ), the whole group will be wait listed. Or, you can reduce the number of people you are registering to %1 to avoid being put on the waiting list.", [1 => $this->_availableRegistrations]);
 
       if ($this->_requireApproval) {
-        $this->_requireApprovalMsg = CRM_Utils_Array::value('approval_req_text', $this->_values['event'],
-          ts('Registration for this event requires approval. Once your registration(s) have been reviewed, you will receive an email with a link to a web page where you can complete the registration process.')
-        );
+        $this->_requireApprovalMsg = $this->_values['event']['approval_req_text'] ??
+          ts('Registration for this event requires approval. Once your registration(s) have been reviewed, you will receive an email with a link to a web page where you can complete the registration process.');
       }
     }
 
@@ -420,9 +408,8 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
     if ($this->_requireApproval &&
       !$this->_allowWaitlist && !$bypassPayment
     ) {
-      $this->_requireApprovalMsg = CRM_Utils_Array::value('approval_req_text', $this->_values['event'],
-        ts('Registration for this event requires approval. Once your registration has been reviewed, you will receive an email with a link to a web page where you can complete the registration process.')
-      );
+      $this->_requireApprovalMsg = $this->_values['event']['approval_req_text'] ??
+        ts('Registration for this event requires approval. Once your registration has been reviewed, you will receive an email with a link to a web page where you can complete the registration process.');
     }
 
     //lets display status to primary page only.
@@ -600,7 +587,7 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
     /// see https://lab.civicrm.org/dev/core/-/issues/5168
     if ($form->getPriceSetID() && !empty($fields['bypass_payment']) && $form->_allowConfirmation) {
       if ($spacesAvailable === 0 ||
-        (empty($fields['priceSetId']) && CRM_Utils_Array::value('additional_participants', $fields) < $spacesAvailable)
+        (empty($fields['priceSetId']) && ($fields['additional_participants'] ?? 0) < $spacesAvailable)
       ) {
         $errors['bypass_payment'] = ts("You have not been added to the waiting list because there are spaces available for this event. We recommend registering yourself for an available space instead.");
       }
@@ -617,7 +604,7 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
       $primaryParticipantCount = $form->getParticipantCount($ppParams);
 
       //get price set fields errors in.
-      $errors = array_merge($errors, CRM_Utils_Array::value(0, $priceSetErrors, []));
+      $errors = array_merge($errors, $priceSetErrors[0] ?? []);
 
       $totalParticipants = $primaryParticipantCount;
       if ($numberAdditionalParticipants) {
@@ -835,7 +822,7 @@ class CRM_Event_Form_Registration_Register extends CRM_Event_Form_Registration {
       $this->set('amount_level', $params['amount_level']);
 
       // generate and set an invoiceID for this transaction
-      $invoiceID = md5(uniqid(rand(), TRUE));
+      $invoiceID = bin2hex(random_bytes(16));
       $this->set('invoiceID', $invoiceID);
 
       if ($this->_paymentProcessor) {
