@@ -84,6 +84,8 @@ class CRM_Financial_BAO_Order {
 
   private array $contributionValues;
 
+  private ?int $existingContributionID = NULL;
+
   /**
    * @param bool $isExcludeExpiredFields
    *
@@ -219,6 +221,18 @@ class CRM_Financial_BAO_Order {
    */
   protected $overrideTotalAmount;
 
+  /**
+   * Override for the total amount of the order exclusive of tax.
+   *
+   * We set this when that is what is known by the form, in order to 'prefer'
+   * the raw form when it comes to forwards / backwards calculations.
+   *
+   * When there is a single line item the order total may be overriden.
+   *
+   * @var float|null
+   */
+  protected ?float $overrideTotalAmountTaxExclusive = NULL;
+
   /**
    * Line items in the order.
    *
@@ -359,6 +373,7 @@ class CRM_Financial_BAO_Order {
       'OverrideTotalAmount' => $this->getOverrideTotalAmount(),
       'OverrideFinancialType' => $this->getOverrideFinancialTypeID(),
       'PriceSelection' => $this->getPriceSelection(),
+      'OverrideTotalAmountTaxExclusive' => $this->getOverrideTotalAmountTaxExclusive(),
     ];
   }
 
@@ -377,7 +392,7 @@ class CRM_Financial_BAO_Order {
   /**
    * Form object - if present the buildAmount hook will be called.
    *
-   * @var \CRM_Member_Form_Membership|\CRM_Member_Form_MembershipRenewal
+   * @var \CRM_Member_Form_Membership|\CRM_Member_Form_MembershipRenewal|\CRM_Contribute_Form_Contribution
    */
   protected $form;
 
@@ -431,6 +446,18 @@ class CRM_Financial_BAO_Order {
     $this->overrideTotalAmount = $overrideTotalAmount;
   }
 
+  public function setOverrideTotalAmountTaxExclusive(float $overrideTotalAmountTaxExclusive): void {
+    $this->overrideTotalAmountTaxExclusive = $overrideTotalAmountTaxExclusive;
+    if ($this->getOverrideFinancialTypeID()) {
+      $taxRate = $this->getTaxRate($this->getOverrideFinancialTypeID());
+      $this->setOverrideTotalAmount($overrideTotalAmountTaxExclusive + ($overrideTotalAmountTaxExclusive * $taxRate));
+    }
+  }
+
+  public function getOverrideTotalAmountTaxExclusive(): ?float {
+    return $this->overrideTotalAmountTaxExclusive;
+  }
+
   /**
    * Get override for total amount.
    *
@@ -766,7 +793,7 @@ class CRM_Financial_BAO_Order {
    */
   public function setPriceSelectionFromUnfilteredInput(array $input): void {
     foreach ($input as $fieldName => $value) {
-      if (strpos($fieldName, 'price_') === 0) {
+      if (str_starts_with($fieldName, 'price_')) {
         $fieldID = substr($fieldName, 6);
         if (is_numeric($fieldID)) {
           $this->priceSelection[$fieldName] = $value;
@@ -897,6 +924,24 @@ class CRM_Financial_BAO_Order {
     return $lines;
   }
 
+  /**
+   * Get line items that specifically relate to participants.
+   *
+   * return array
+   *
+   * @throws \CRM_Core_Exception
+   */
+  public function getParticipantLineItems():array {
+    $lines = $this->getLineItems();
+    foreach ($lines as $index => $line) {
+      if ($line['entity_table'] !== 'civicrm_participant') {
+        unset($lines[$index]);
+        continue;
+      }
+    }
+    return $lines;
+  }
+
   /**
    * Get an array of all membership types included in the order.
    *
@@ -956,6 +1001,17 @@ class CRM_Financial_BAO_Order {
         $this->setPriceSetID($lineItems[0]['price_field_id.price_set_id']);
       }
     }
+    elseif ($this->getExistingContributionID()) {
+      $lineItems = $this->getLinesForContribution();
+      // Set the price set ID from the first line item (we need to set this here
+      // to prevent a loop later when we retrieve the price field metadata to
+      // set the 'title' (as accessed from workflow message templates).
+      // Contributions *should* all have line items, but historically, imports did not create them.
+      if ($lineItems) {
+        $firstItem = reset($lineItems);
+        $this->setPriceSetID($firstItem['price_field_id.price_set_id']);
+      }
+    }
     else {
       foreach ($this->getPriceOptions() as $fieldID => $valueID) {
         if ($valueID !== '') {
@@ -1309,14 +1365,28 @@ class CRM_Financial_BAO_Order {
     $lineItem['tax_rate'] = $taxRate = $this->getTaxRate($financialTypeID);
     if ($taxRate) {
       // Total is tax inclusive.
-      $lineItem['tax_amount'] = ($taxRate / 100) * $this->getOverrideTotalAmount() / (1 + ($taxRate / 100));
-      $lineItem['line_total'] = $this->getOverrideTotalAmount() - $lineItem['tax_amount'];
+      $taxExclusiveAmount = $this->getOverrideTotalAmountTaxExclusive();
+      if ($taxExclusiveAmount) {
+        $lineItem['line_total'] = $taxExclusiveAmount;
+        $lineItem['tax_amount'] = ($taxRate / 100) * $taxExclusiveAmount;
+        // Set to 1 for consistency with historical behaviour on the only form that calls this section.
+        // There may be a case to set unit_price to 1 & qty to x
+        $lineItem['qty'] = 1;
+      }
+      else {
+        $lineItem['tax_amount'] = ($taxRate / 100) * $this->getOverrideTotalAmount() / (1 + ($taxRate / 100));
+        $lineItem['line_total'] = $this->getOverrideTotalAmount() - $lineItem['tax_amount'];
+      }
     }
     else {
-      $lineItem['line_total'] = $this->getOverrideTotalAmount();
+      $lineItem['line_total'] = $this->getOverrideTotalAmountTaxExclusive() ?: $this->getOverrideTotalAmount();
       $lineItem['tax_amount'] = 0.0;
       $lineItem['line_total_inclusive'] = $lineItem['line_total'];
     }
+    if ($this->getExistingContributionID() && $lineItem['unit_price'] === 1.0) {
+      // Perhaps the existing contribution ID check can go...
+      $lineItem['qty'] = $lineItem['line_total'];
+    }
     if (!empty($lineItem['qty'])) {
       $lineItem['unit_price'] = $lineItem['line_total'] / $lineItem['qty'];
     }
@@ -1333,7 +1403,8 @@ class CRM_Financial_BAO_Order {
    * @throws \CRM_Core_Exception
    */
   protected function getLinesFromTemplateContribution(): array {
-    $lines = $this->getLinesForContribution();
+    // Rekey the array to a 0 index with array_merge.
+    $lines = array_merge($this->getLinesForContribution());
     foreach ($lines as &$line) {
       // The apiv4 insists on adding id - so let it get all the details
       // and we will filter out those that are not part of a template here.
@@ -1369,7 +1440,7 @@ class CRM_Financial_BAO_Order {
    */
   protected function getLinesForContribution(): array {
     return (array) LineItem::get(FALSE)
-      ->addWhere('contribution_id', '=', $this->getTemplateContributionID())
+      ->addWhere('contribution_id', '=', $this->getExistingContributionID() ?: $this->getTemplateContributionID())
       ->setSelect([
         'contribution_id',
         'entity_id',
@@ -1388,7 +1459,7 @@ class CRM_Financial_BAO_Order {
         'participant_count',
         'membership_num_terms',
       ])
-      ->execute();
+      ->execute()->indexBy('id');
   }
 
   /**
@@ -1474,7 +1545,7 @@ class CRM_Financial_BAO_Order {
     $contributionValues['amount_level'] = $this->getAmountLevel();
     $contributionValues['contribution_status_id:name'] = 'Pending';
     $contributionValues['line_item'] = [$this->getLineItems()];
-    return Contribution::create()
+    return Contribution::create(FALSE)
       ->setValues($contributionValues)->execute();
   }
 
@@ -1524,4 +1595,12 @@ class CRM_Financial_BAO_Order {
     return civicrm_api4($entity, 'save', ['records' => [$entityValues]])->first()['id'];
   }
 
+  public function getExistingContributionID(): ?int {
+    return $this->existingContributionID;
+  }
+
+  public function setExistingContributionID(?int $existingContributionID): void {
+    $this->existingContributionID = $existingContributionID;
+  }
+
 }
