@@ -880,7 +880,7 @@ ORDER BY   civicrm_email.is_bulkmail DESC
     $emailDomain = CRM_Core_BAO_MailSettings::defaultDomain();
     // Make sure the user configured the site correctly, otherwise you just get "Could not identify any recipients. Perhaps the group is empty?" from the mailing UI
     if (empty($emailDomain)) {
-      CRM_Core_Error::debug_log_message('Error setting verp parameters, defaultDomain is NULL.  Did you configure the bounce processing account for this domain?');
+      Civi::log()->error('Error setting verp parameters, defaultDomain is NULL.  Did you configure the bounce processing account for this domain?');
     }
 
     foreach ($verpTokens as $key => $value) {
@@ -906,7 +906,6 @@ ORDER BY   civicrm_email.is_bulkmail DESC
     // Generating URLs is expensive, so we only call it once for each of these 5 URLs.
     $genericURL = CRM_Utils_System::url('civicrm/mailing/genericUrlPath', "reset=1&jid={$job_id}&qid={$event_queue_id}&h={$hash}", TRUE, NULL, TRUE, TRUE);
     $urls = [
-      'forward' => str_replace('genericUrlPath', 'forward', $genericURL),
       'unsubscribeUrl' => str_replace('genericUrlPath', 'unsubscribe', $genericURL),
       'resubscribeUrl' => str_replace('genericUrlPath', 'resubscribe', $genericURL),
       'optOutUrl' => str_replace('genericUrlPath', 'optout', $genericURL),
@@ -1259,7 +1258,6 @@ ORDER BY   civicrm_email.is_bulkmail DESC
       'reply' => CRM_Mailing_Event_BAO_MailingEventReply::getTableName(),
       'unsubscribe' => CRM_Mailing_Event_BAO_MailingEventUnsubscribe::getTableName(),
       'bounce' => CRM_Mailing_Event_BAO_MailingEventBounce::getTableName(),
-      'forward' => CRM_Mailing_Event_BAO_MailingEventForward::getTableName(),
       'url' => CRM_Mailing_BAO_MailingTrackableURL::getTableName(),
       'urlopen' => CRM_Mailing_Event_BAO_MailingEventTrackableURLOpen::getTableName(),
       'component' => CRM_Mailing_BAO_MailingComponent::getTableName(),
@@ -1353,6 +1351,7 @@ ORDER BY   civicrm_email.is_bulkmail DESC
       if (isset($mailing->group_id)) {
         $row['id'] = $mailing->group_id;
         $row['name'] = $mailing->group_title;
+        $row['mailing'] = FALSE;
         $row['link'] = CRM_Utils_System::url('civicrm/group/search',
           "reset=1&force=1&context=smog&gid={$row['id']}"
         );
@@ -1388,7 +1387,6 @@ ORDER BY   civicrm_email.is_bulkmail DESC
                             COUNT(DISTINCT {$t['queue']}.id) as queue,
                             COUNT(DISTINCT {$t['delivered']}.id) as delivered,
                             COUNT(DISTINCT {$t['reply']}.id) as reply,
-                            COUNT(DISTINCT {$t['forward']}.id) as forward,
                             COUNT(DISTINCT {$t['bounce']}.id) as bounce,
                             COUNT(DISTINCT {$t['urlopen']}.id) as url,
                             COUNT(DISTINCT civicrm_mailing_spool.id) as spool
@@ -1397,8 +1395,6 @@ ORDER BY   civicrm_email.is_bulkmail DESC
                     ON      {$t['queue']}.job_id = civicrm_mailing_job.id
             LEFT JOIN       {$t['reply']}
                     ON      {$t['reply']}.event_queue_id = {$t['queue']}.id
-            LEFT JOIN       {$t['forward']}
-                    ON      {$t['forward']}.event_queue_id = {$t['queue']}.id
             LEFT JOIN       {$t['bounce']}
                     ON      {$t['bounce']}.event_queue_id = {$t['queue']}.id
             LEFT JOIN       {$t['delivered']}
@@ -1416,10 +1412,10 @@ ORDER BY   civicrm_email.is_bulkmail DESC
     $report['event_totals'] = [];
     $path = 'civicrm/mailing/report/event';
     $elements = [
+      'recipients',
       'queue',
       'delivered',
       'url',
-      'forward',
       'reply',
       'unsubscribe',
       'optout',
@@ -1489,7 +1485,6 @@ ORDER BY   civicrm_email.is_bulkmail DESC
         'delivered' => CRM_Utils_System::url($path, "$arg&event=delivered"),
         'bounce' => CRM_Utils_System::url($path, "$arg&event=bounce"),
         'unsubscribe' => CRM_Utils_System::url($path, "$arg&event=unsubscribe"),
-        'forward' => CRM_Utils_System::url($path, "$arg&event=forward"),
         'reply' => CRM_Utils_System::url($path, "$arg&event=reply"),
         'opened' => CRM_Utils_System::url($path, "$arg&event=opened"),
       ];
@@ -1500,9 +1495,7 @@ ORDER BY   civicrm_email.is_bulkmail DESC
       $report['jobs'][] = $row;
     }
 
-    if (empty($report['event_totals']['queue'])) {
-      $report['event_totals']['queue'] = CRM_Mailing_BAO_MailingRecipients::mailingSize($mailing_id);
-    }
+    $report['event_totals']['recipients'] = CRM_Mailing_BAO_MailingRecipients::mailingSize($mailing_id);
 
     if (!empty($report['event_totals']['queue'])) {
       $report['event_totals']['delivered_rate'] = (100.0 * $report['event_totals']['delivered']) / $report['event_totals']['queue'];
@@ -1562,7 +1555,6 @@ ORDER BY   civicrm_email.is_bulkmail DESC
       'bounce' => CRM_Utils_System::url($path, "$arg&event=bounce"),
       'unsubscribe' => CRM_Utils_System::url($path, "$arg&event=unsubscribe"),
       'optout' => CRM_Utils_System::url($path, "$arg&event=optout"),
-      'forward' => CRM_Utils_System::url($path, "$arg&event=forward"),
       'reply' => CRM_Utils_System::url($path, "$arg&event=reply"),
       'opened' => CRM_Utils_System::url($path, "$arg&event=opened"),
     ];
@@ -1575,7 +1567,7 @@ ORDER BY   civicrm_email.is_bulkmail DESC
     $action = array_sum(array_keys($actionLinks));
 
     $report['event_totals']['actionlinks'] = [];
-    foreach (['clicks', 'clicks_unique', 'queue', 'delivered', 'bounce', 'unsubscribe', 'forward', 'reply', 'opened', 'opened_unique', 'optout'] as $key) {
+    foreach (['clicks', 'clicks_unique', 'queue', 'delivered', 'bounce', 'unsubscribe', 'reply', 'opened', 'opened_unique', 'optout'] as $key) {
       $url = 'mailing/detail';
       $reportFilter = "reset=1&mailing_id_value={$mailing_id}";
       $searchFilter = "force=1&mailing_id=%%mid%%";
@@ -1590,11 +1582,6 @@ ORDER BY   civicrm_email.is_bulkmail DESC
           $searchFilter .= "&mailing_delivery_status=N";
           break;
 
-        case 'forward':
-          $reportFilter .= "&is_forwarded_value=1";
-          $searchFilter .= "&mailing_forward=1";
-          break;
-
         case 'reply':
           $reportFilter .= "&is_replied_value=1";
           $searchFilter .= "&mailing_reply_status=Y";
@@ -2502,6 +2489,8 @@ ORDER BY civicrm_mailing.id DESC";
       $types = [];
       $types[] = [
         'name' => 'traditional',
+        'label' => ts('Traditional'),
+        'description' => ts('Standard CiviMail interface with wysiwyg editor.'),
         'editorUrl' => CRM_Mailing_Info::workflowEnabled() ? '~/crmMailing/EditMailingCtrl/workflow.html' : '~/crmMailing/EditMailingCtrl/2step.html',
         'weight' => 0,
       ];
@@ -2526,17 +2515,21 @@ ORDER BY civicrm_mailing.id DESC";
   }
 
   /**
-   * Get a list of template types.
+   * Pseudoconstant callback for `template_type` field.
    *
    * @return array
-   *   Array(string $name => string $label).
    */
-  public static function getTemplateTypeNames() {
-    $r = [];
+  public static function getTemplateTypeNames(): array {
+    $types = [];
     foreach (self::getTemplateTypes() as $type) {
-      $r[$type['name']] = $type['name'];
+      $types[] = [
+        'id' => $type['name'],
+        'name' => $type['name'],
+        'label' => $type['label'] ?? ucfirst($type['name']),
+        'description' => $type['description'] ?? NULL,
+      ];
     }
-    return $r;
+    return $types;
   }
 
 }
