@@ -148,7 +148,7 @@ class CRM_Report_Form_Contribute_Repeat extends CRM_Report_Form {
           'on_hold' => [
             'title' => ts('On Hold'),
             'type' => CRM_Utils_Type::T_INT,
-            'operatorType' => CRM_Report_Form::OP_MULTISELECT,
+            'operatorType' => CRM_Report_Form::OP_SELECT,
             'options' => ['' => ts('Any')] + CRM_Core_PseudoConstant::emailOnHoldOptions(),
           ],
         ],
@@ -228,7 +228,7 @@ class CRM_Report_Form_Contribute_Repeat extends CRM_Report_Form {
             'title' => ts('Financial Type'),
             'type' => CRM_Utils_Type::T_INT,
             'operatorType' => CRM_Report_Form::OP_MULTISELECT,
-            'options' => CRM_Contribute_BAO_Contribution::buildOptions('financial_type_id', 'search'),
+            'options' => CRM_Contribute_BAO_Contribution::buildOptions('financial_type_id', 'search', ['orderColumn' => 'weight']),
           ],
           'contribution_status_id' => [
             'title' => ts('Contribution Status'),
