@@ -52,7 +52,7 @@ class CRM_Report_Utils_Report {
 
     if ($optionVal) {
       $templateInfo = CRM_Core_OptionGroup::getRowValues('report_template', "{$optionVal}", 'value');
-      return [CRM_Utils_Array::value('id', $templateInfo), $optionVal];
+      return [$templateInfo['id'] ?? NULL, $optionVal];
     }
 
     return FALSE;
@@ -191,7 +191,7 @@ WHERE  inst.report_id = %1";
     if (empty($instanceInfo['attachments'])) {
       $instanceInfo['attachments'] = [];
     }
-    $params['attachments'] = array_merge(CRM_Utils_Array::value('attachments', $instanceInfo), $attachments);
+    $params['attachments'] = array_merge($instanceInfo['attachments'] ?? [], $attachments);
     $params['text'] = '';
     $params['html'] = $fileContent;
 
@@ -388,11 +388,11 @@ WHERE  inst.report_id = %1";
 
     // hack for now, CRM-8358
     $_REQUEST['instanceId'] = $instanceId;
-    $_REQUEST['sendmail'] = CRM_Utils_Array::value('sendmail', $params, 1);
+    $_REQUEST['sendmail'] = $params['sendmail'] ?? 1;
 
     // if cron is run from terminal --output is reserved, and therefore we would provide another name 'format'
     $_REQUEST['output'] = $params['format'] ?? $params['output'] ?? 'pdf';
-    $_REQUEST['reset'] = CRM_Utils_Array::value('reset', $params, 1);
+    $_REQUEST['reset'] = $params['reset'] ?? 1;
 
     $optionVal = self::getValueFromUrl($instanceId);
     $messages = ['Report Mail Triggered...'];
