@@ -499,17 +499,17 @@ class CRM_Upgrade_Incremental_MessageTemplates {
 
   /**
    * Get the upgrade messages.
+   *
+   * @fromVer version we are upgrading from
    */
-  public function getUpgradeMessages() {
+  public function getUpgradeMessages($fromVer) {
     $updates = $this->getTemplatesToUpdate();
 
+    $uneditedTemplates = [];
     // workflow_name is not available pre-5.26, so we won't check if templates are changed or not pre-5.26
-    try {
+    if (version_compare($fromVer, 5.26, '>=')) {
       $uneditedTemplates = $this->getUneditedTemplates();
     }
-    catch (Exception $e) {
-      $uneditedTemplates = [];
-    }
 
     $messages = [];
     $templateLabel = '';
@@ -542,7 +542,14 @@ class CRM_Upgrade_Incremental_MessageTemplates {
       $workFlowID = CRM_Core_DAO::singleValueQuery('SELECT MAX(id) as id FROM civicrm_option_value WHERE name = %1', [
         1 => [$template['name'], 'String'],
       ]);
-      $content = file_get_contents(\Civi::paths()->getPath('[civicrm.root]/xml/templates/message_templates/' . $template['name'] . '_' . $template['type'] . '.tpl'));
+      if ($template['type'] === 'text') {
+        // We no longer ship text templates.
+        $content = '';
+      }
+      else {
+        $content = file_get_contents(\Civi::paths()
+          ->getPath('[civicrm.root]/xml/templates/message_templates/' . $template['name'] . '_' . $template['type'] . '.tpl'));
+      }
       $templatesToUpdate = [];
       if (!empty($workFlowID)) {
         // This could be empty if the template was deleted. It should not happen,
