@@ -267,7 +267,7 @@ class CRM_Utils_System_Backdrop extends CRM_Utils_System_DrupalBase {
    */
   public function mapConfigToSSL() {
     global $base_url;
-    $base_url = str_replace('http://', 'https://', $base_url);
+    $base_url = str_replace('http://', 'https://', (string) $base_url);
   }
 
   /**
@@ -520,14 +520,6 @@ AND    u.status = 1
     return $user->uid;
   }
 
-  /**
-   * @inheritDoc
-   */
-  public function logout() {
-    module_load_include('inc', 'user', 'user.pages');
-    user_logout();
-  }
-
   /**
    * Get the default location for CiviCRM blocks.
    *
@@ -567,7 +559,7 @@ AND    u.status = 1
     }
 
     // For Backdrop multi-site CRM-11313
-    if ($realPath && strpos($realPath, 'sites/all/modules/') === FALSE) {
+    if ($realPath && !str_contains($realPath, 'sites/all/modules/')) {
       preg_match('@sites/([^/]*)/modules@s', $realPath, $matches);
       if (!empty($matches[1])) {
         $_SERVER['HTTP_HOST'] = $matches[1];
@@ -1021,7 +1013,7 @@ AND    u.status = 1
     // Handle absolute urls
     // compares $url (which is some unknown/untrusted value from a third-party dev) to the CMS's base url (which is independent of civi's url)
     // to see if the url is within our Backdrop dir, if it is we are able to treated it as an internal url
-    if (strpos($url, $base_url) === 0) {
+    if (str_starts_with($url, $base_url)) {
       $file = trim(str_replace($base_url, '', $url), '/');
       // CRM-18130: Custom CSS URL not working if aliased or rewritten
       if (file_exists(BACKDROP_ROOT . $file)) {
@@ -1030,7 +1022,7 @@ AND    u.status = 1
       }
     }
     // Handle relative urls that are within the CiviCRM module directory
-    elseif (strpos($url, $base) === 0) {
+    elseif (str_starts_with($url, $base)) {
       $internal = TRUE;
       $url = $this->appendCoreDirectoryToResourceBase(dirname(backdrop_get_path('module', 'civicrm')) . '/') . trim(substr($url, strlen($base)), '/');
     }
@@ -1153,7 +1145,11 @@ AND    u.status = 1
    * CMS's drupal views expectations, if any.
    */
   public function getCRMDatabasePrefix(): string {
-    return str_replace('`', '', parent::getCRMDatabasePrefix());
+    $crmDatabaseName = parent::getCRMDatabaseName();
+    if (!empty($crmDatabaseName)) {
+      return "$crmDatabaseName.";
+    }
+    return $crmDatabaseName;
   }
 
   /**
@@ -1212,16 +1208,15 @@ AND    u.status = 1
    * @inheritdoc
    */
   public function theme(&$content, $print = FALSE, $maintenance = FALSE) {
+    if ($maintenance) {
+      \CRM_Core_Error::deprecationMessage('CRM_Utils_System::theme called with $maintenance = TRUE - please use renderMaintenanceMessage instead');
+    }
+
     $ret = FALSE;
 
     if (!$print) {
       if ($maintenance) {
-        backdrop_set_breadcrumb('');
-        backdrop_maintenance_theme();
-        if ($region = CRM_Core_Region::instance('html-header', FALSE)) {
-          CRM_Utils_System::addHTMLHead($region->render(''));
-        }
-        print theme('maintenance_page', ['content' => $content]);
+        print $this->renderMaintanceMessage($content);
         exit();
       }
       $ret = TRUE;
@@ -1237,6 +1232,18 @@ AND    u.status = 1
     }
   }
 
+  /**
+   * @inheritdoc
+   */
+  public function renderMaintanceMessage(string $content): string {
+    backdrop_set_breadcrumb('');
+    backdrop_maintenance_theme();
+    if ($region = CRM_Core_Region::instance('html-header', FALSE)) {
+      $this->addHTMLHead($region->render(''));
+    }
+    return theme('maintenance_page', ['content' => $content]);
+  }
+
   /**
    * @inheritdoc
    */
@@ -1247,4 +1254,8 @@ AND    u.status = 1
     return function_exists('ip_address') ? ip_address() : ($_SERVER['REMOTE_ADDR'] ?? NULL);
   }
 
+  public function isMaintenanceMode(): bool {
+    return state_get('maintenance_mode', FALSE);
+  }
+
 }
