@@ -153,7 +153,7 @@
       $scope.$ctrl = this;
     })
 
-    .factory('searchMeta', function($q, crmApi4, formatForSelect2) {
+    .factory('searchMeta', function($q, crmApi4, formatForSelect2, md5) {
       function getEntity(entityName) {
         if (entityName) {
           return _.find(CRM.crmSearchAdmin.schema, {name: entityName});
@@ -425,6 +425,16 @@
           }
           return searchTasks[entityName];
         },
+        createSqlName: function(key) {
+          // Generate a preview of the default SQL column-names that would be generated by the server.
+          // WARNING: This formula lives in both Civi\Search\Meta and crmSearchAdmin.module.js. Keep synchronized!
+          let [name] = key.split(':');
+          if (name.length <= 58) {
+            return CRM.utils.munge(name, '_');
+          } else {
+            return CRM.utils.munge(name, '_', 42) + (md5(name)).substring(0, 16);
+          }
+        },
         // Supply default aggregate function appropriate to the data_type
         getDefaultAggregateFn: function(info, apiParams) {
           let arg = info.args[0] || {};
